//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

//	ChatSelector.cpp : implementation file
//

#include "stdafx.h"
#include "emule.h"
#include "ChatSelector.h"
#include "packets.h"
#include "otherfunctions.h"
#include "Smtp.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#define new DEBUG_NEW
#endif


CChatItem::CChatItem()
{
	m_pClient = NULL;
	m_pLog = NULL;
	m_strMessagePending.Empty();
	m_bNotify = false;
	m_iHistoryIndex = 0;
}

//	CChatSelector

IMPLEMENT_DYNAMIC(CChatSelector, CClosableTabCtrl)
CChatSelector::CChatSelector()
{
	m_pCloseBtn = NULL;
	m_pMessageBox = NULL;
	m_pSendBtn = NULL;
	lastemptyicon = false;
	blinkstate = false;
	m_Timer = 0;
}

CChatSelector::~CChatSelector()
{
}


BEGIN_MESSAGE_MAP(CChatSelector, CClosableTabCtrl)
	ON_WM_TIMER()
	ON_NOTIFY_REFLECT(TCN_SELCHANGE, OnTcnSelchangeChatsel)
	ON_WM_SIZE()
	ON_BN_CLICKED(IDC_CCLOSE, OnBnClickedCclose)
	ON_BN_CLICKED(IDC_CSEND, OnBnClickedCsend)
	ON_NOTIFY(EN_LINK, 124, OnEnLinkMessageBox)
	ON_WM_DESTROY()
END_MESSAGE_MAP()

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	CChatSelector message handlers
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CChatSelector::Init()
{
	static const uint16 s_auIconResID[] =
	{
		IDI_CHAT,
		IDI_MESSAGE,
		IDI_MPENDING
	};

	CRect rect, rClose, rSend, rMessage, rChatOut;

	GetClientRect(&rect);
	AdjustRect(FALSE, rect);

	m_pCloseBtn = GetParent()->GetDlgItem(IDC_CCLOSE);
	m_pCloseBtn->SetParent(this);
	m_pCloseBtn->GetWindowRect(&rClose);
	m_pCloseBtn->SetWindowPos( NULL, rect.right - 7 - rClose.Width(), rect.bottom - 7 - rClose.Height(),
	                           rClose.Width(), rClose.Height(), SWP_NOZORDER );
	m_pCloseBtn->EnableWindow(false);

	m_pSendBtn = GetParent()->GetDlgItem(IDC_CSEND);
	m_pSendBtn->SetParent(this);
	m_pSendBtn->GetWindowRect(&rSend);
	m_pSendBtn->SetWindowPos( NULL, rect.right - 7 - rClose.Width() - 7 - rSend.Width(), rect.bottom - 7 - rSend.Height(),
	                          rSend.Width(), rSend.Height(), SWP_NOZORDER );
	m_pSendBtn->EnableWindow(false);

	m_pMessageBox = GetParent()->GetDlgItem(IDC_CMESSAGE);
	m_pMessageBox->SetParent(this);
	m_pMessageBox->GetWindowRect(&rMessage);
	m_pMessageBox->SetWindowPos( NULL, rect.left + 7, rect.bottom - 9 - rMessage.Height(),
	                             rect.right - 7 - rClose.Width() - 7 - rSend.Width() - 21,
	                             rMessage.Height(), SWP_NOZORDER );

	int iTop = rClose.Height() > rSend.Height() ? rClose.Height() : rSend.Height();

	if (iTop < rMessage.Height())
		iTop = rMessage.Height();

	rChatOut = rect;
	rChatOut.top += 7;
	rChatOut.left += 7;
	rChatOut.right -= 18;
	rChatOut.bottom -= 7 - iTop - 7;

	ModifyStyle(0, WS_CLIPCHILDREN);
	chatout.Create(WS_VISIBLE | WS_CHILD | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_HSCROLL | WS_VSCROLL | ES_MULTILINE | ES_READONLY | ES_NOHIDESEL, rChatOut, this, 124);
	chatout.ModifyStyleEx(0, WS_EX_STATICEDGE, SWP_FRAMECHANGED);
	chatout.SetEventMask(chatout.GetEventMask() | ENM_LINK);
	chatout.SetFont(&g_eMuleApp.m_pdlgEmule->m_fontDefault);
	chatout.SetTitle(GetResString(IDS_CW_MESSAGES));
	chatout.AppendText(CLIENT_NAME, 0x1000000, 0x1000000, HTC_LINK);
	chatout.AppendText(CString(" Version " CURRENT_VERSION_LONG " - ") + GetResString(IDS_CHAT_WELCOME));

	m_imageList.Create(16, 16, ILC_COLOR32 | ILC_MASK, 0, 10);
	FillImgLstWith16x16Icons(&m_imageList, s_auIconResID, ELEMENT_COUNT(s_auIconResID));
	SetImageList(&m_imageList);

	VERIFY((m_Timer = SetTimer(20, 1500, 0)));
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CChatItem* CChatSelector::StartSession(CUpDownClient* pClient)
{
	if (pClient == NULL)
		return NULL;

	EMULE_TRY

	if (GetTabByClient(pClient) != 0xFFFF)
	{
		SetCurSel(GetTabByClient(pClient));
		ShowChat();
		return NULL;
	}

	CChatItem		*pChatItem = new CChatItem();

	pChatItem->m_pClient = pClient;
	pChatItem->m_pLog = new CHTRichEditCtrl;

	CRect rcChat, rcClose, rcSend, rcMessage, rcClient;

	m_pCloseBtn->GetWindowRect(&rcClose);
	m_pSendBtn->GetWindowRect(&rcSend);
	m_pMessageBox->GetWindowRect(&rcMessage);

	int iTop = rcClose.Height() > rcSend.Height() ? rcClose.Height() : rcSend.Height();

	if (iTop < rcMessage.Height())
		iTop = rcMessage.Height();
	
	GetClientRect(&rcClient);
	AdjustRect(false, rcClient);
	rcChat.left = rcClient.left + 7;
	rcChat.top = rcClient.top + 7;
	rcChat.right = rcChat.left + rcClient.right - 18;
	rcChat.bottom = rcChat.top + rcClient.Height() - 7 - iTop - 14;
	if (GetItemCount() == 0)
		rcChat.top += 19;

	pChatItem->m_pLog->Create(WS_CHILD | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_HSCROLL | WS_VSCROLL | ES_MULTILINE | ES_READONLY | ES_NOHIDESEL, rcChat, this, (UINT)-1);
	pChatItem->m_pLog->ModifyStyleEx(0, WS_EX_STATICEDGE, SWP_FRAMECHANGED);
	pChatItem->m_pLog->SendMessage(EM_SETMARGINS, EC_LEFTMARGIN | EC_RIGHTMARGIN, MAKELONG(3, 3));
	pChatItem->m_pLog->SetEventMask(pChatItem->m_pLog->GetEventMask() | ENM_LINK);
	pChatItem->m_pLog->SetFont(&g_eMuleApp.m_pdlgEmule->m_fontDefault);
	pChatItem->m_pLog->m_bWordWrap = true;
	pChatItem->m_pLog->SetTargetDevice(NULL, 0);

	COleDateTime	timelog(COleDateTime::GetCurrentTime());

	pChatItem->m_pLog->AppendText(GetResString(IDS_CHAT_START) + CString(pClient->GetUserName()) + _T(" (IP: ") + CString(pClient->GetFullIP()) + _T(") - ") + timelog.Format() + _T("\n"), RGB(255, 0, 0));
	pClient->SetChatState(MS_CHATTING);

	CString			strName = pClient->GetUserName();

	if (strName.GetLength() > 30)
	{
		strName.Truncate(30);
		strName += _T("...");
	}
	else if (strName.IsEmpty())
		strName.Format(_T("(%s)"), GetResString(IDS_UNKNOWN));

	TCITEM		tcitem;

	tcitem.mask = TCIF_PARAM | TCIF_TEXT | TCIF_IMAGE;
	tcitem.lParam = (LPARAM)pChatItem;
	tcitem.pszText = strName.GetBuffer();
	tcitem.cchTextMax = strName.GetLength() + 1;
	tcitem.iImage = 0;
	
	int			iResult = InsertItem(GetItemCount(), &tcitem);

	m_pCloseBtn->EnableWindow(true);
	m_pSendBtn->EnableWindow(true);

	if (iResult != -1 && IsWindowVisible())
	{
		SetCurSel(iResult);
		pChatItem->m_pLog->SetTitle(pClient->GetUserName());
		ShowChat();
	}

	return pChatItem;

	EMULE_CATCH

	return NULL;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
uint16 CChatSelector::GetTabByClient(CUpDownClient* client)
{
	TCITEM tcitem;

	for (int i = 0; i < GetItemCount(); i++)
	{
		tcitem.mask = TCIF_PARAM;
		if (GetItem(i, &tcitem) && ((CChatItem*)tcitem.lParam)->m_pClient == client)
			return i;
	}
	return (uint16)-1;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CChatItem* CChatSelector::GetItemByClient(CUpDownClient* client)
{
	TCITEM	tcitem;

	for (int i = 0; i < GetItemCount(); i++)
	{
		tcitem.mask = TCIF_PARAM;
		if (GetItem(i, &tcitem) && ((CChatItem*)tcitem.lParam)->m_pClient == client)
			return(CChatItem*)tcitem.lParam;
	}
	return NULL;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CChatSelector::ProcessMessage(CUpDownClient* sender, LPCTSTR message)
{
	CString		strMessage = CString(message).MakeLower();
	CString		strResToken;
	int			iCurPos = 0;

	//filter me?
	if ((strMessage.Find(_T("emule sans upload @ http://www.chez.com/theworld/")) != -1) || (strMessage.Find(_T("http://fullspeed.to/mison")) != -1) && (g_eMuleApp.m_pGlobPrefs->IsCounterMeasures()))
	{
		AddDebugLogLine(false, RGB_LOG_DIMMED + _T("Anti-leechermods: Client '%s' (%s) has been banned because of spamming"), sender->GetUserName(), GetClientNameAndVersionString(sender));
		sender->Ban(_T("AUTOMATED MESSAGE: Your client is being banned for spamming and leeching!"), GetResString(IDS_BAN_LEECHERSPAM));
		return;
	}

	strResToken = g_eMuleApp.m_pGlobPrefs->GetMessageFilter().Tokenize(_T("|"), iCurPos);
	while (!strResToken.IsEmpty())
	{
		if (strMessage.Find(strResToken.MakeLower()) != -1)
		{
			AddDebugLogLine(false, RGB_LOG_DIMMED + _T("Filtered message %s from client '%s' (%s)"), message, sender->GetUserName(), GetClientNameAndVersionString(sender));
			return;
		}
		strResToken = g_eMuleApp.m_pGlobPrefs->GetMessageFilter().Tokenize(_T("|"), iCurPos);
	};

//	continue
	CChatItem		*pChatItem = GetItemByClient(sender);
	bool			bIsNewChatWindow = false;

	if (pChatItem == NULL)
	{
		if (GetItemCount() >= 50)
		{
			AddDebugLogLine(false, RGB_LOG_WARNING_TXT _T("Instant Messaging: Messages limit reached"));		
			return;
		}

		if (g_eMuleApp.m_pGlobPrefs->GetAcceptMessagesFrom() == 4) //no messages
			return;

		if ((g_eMuleApp.m_pGlobPrefs->GetAcceptMessagesFrom() == 2) && (!(sender->IsFriend()))) //only friends
			return;

		if ((g_eMuleApp.m_pGlobPrefs->GetAcceptMessagesFrom() == 3) && (!(sender->IsFriend()))) //log non friends
		{
			AddLogLine(false, GetResString(IDS_IM_MSGFROMCHAT) + _T(" %s: %s"), sender->GetUserName(), message);
			sender->SetChatState(MS_NONE);
			return;
		}

		pChatItem = StartSession(sender);
		if (pChatItem == NULL)
			return;
		bIsNewChatWindow = true;
	}
	COleDateTime	timelog(COleDateTime::GetCurrentTime());

	strMessage.Format(_T("%s (%s): "), sender->GetUserName(), timelog.Format());
	pChatItem->m_pLog->AppendText(strMessage, RGB(50, 200, 250));
	pChatItem->m_pLog->AppendText(CString(message) + _T('\n'));

	if (g_eMuleApp.m_pGlobPrefs->GetAwayState() == true)
	{
		if ((::GetTickCount() - pChatItem->m_pClient->GetAwayMessageResendCount()) > 3000)
		{ //send again only if 3 secs from last away message
			SendAwayMessage(pChatItem);
			pChatItem->m_pClient->SetAwayMessageResendCount(::GetTickCount());
		}
	}

	if (GetCurSel() != GetTabByClient(sender))
		SetItemState(GetTabByClient(sender), TCIS_HIGHLIGHTED, TCIS_HIGHLIGHTED);
	else if (g_eMuleApp.m_pGlobPrefs->GetAwayState() == false && !GetParent()->IsWindowVisible())
	{
		pChatItem->m_bNotify = true;
		if (bIsNewChatWindow || g_eMuleApp.m_pGlobPrefs->GetNotifierPopsEveryChatMsg())
		{
			strMessage.Format(_T("%s %s:'%s'\n"), GetResString(IDS_TBN_NEWCHATMSG), sender->GetUserName(), message);
			g_eMuleApp.m_pSMTPConnection->SendMail(strMessage, g_eMuleApp.m_pGlobPrefs->GetNotifierPopsEveryChatMsg(), g_eMuleApp.m_pGlobPrefs->IsSMTPInfoEnabled());
			g_eMuleApp.m_pdlgEmule->ShowNotifier(strMessage, TBN_CHAT, false, g_eMuleApp.m_pGlobPrefs->GetNotifierPopsEveryChatMsg());
		}
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CChatSelector::SendMessage(const CString& message)
{
	CChatItem		*pChatItem = GetCurrentChatItem();

	if (pChatItem == NULL || pChatItem->m_pClient == NULL)
		return false;

	if (pChatItem->m_strHistoryArray.GetCount() == g_eMuleApp.m_pGlobPrefs->GetMaxChatHistoryLines())
		pChatItem->m_strHistoryArray.RemoveAt(0);
	pChatItem->m_strHistoryArray.Add(CString(message));
	pChatItem->m_iHistoryIndex = pChatItem->m_strHistoryArray.GetCount();

	if (pChatItem->m_pClient->GetChatState() == MS_CONNECTING)
		return false;

#ifdef OLD_SOCKETS_ENABLED
	if (pChatItem->m_pClient->m_pRequestSocket && pChatItem->m_pClient->m_pRequestSocket->IsConnected())
	{
		SendMessagePacket(*pChatItem, message);

		COleDateTime	timelog(COleDateTime::GetCurrentTime());
		CString			strTmp;

		strTmp.Format(_T("%s (%s): "), g_eMuleApp.m_pGlobPrefs->GetUserNick(), timelog.Format());
		pChatItem->m_pLog->AppendText(strTmp, RGB(1, 180, 20));
		pChatItem->m_pLog->AppendText(message + _T('\n'));
	}
	else
	{
		pChatItem->m_pLog->AppendText(_T("*** ") + GetResString(IDS_CONNECTING), RGB(255, 0, 0));
		pChatItem->m_strMessagePending = message;
		pChatItem->m_pClient->SetChatState(MS_CONNECTING);
		pChatItem->m_pClient->TryToConnect();
	}
#endif //OLD_SOCKETS_ENABLED
	return true;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CChatSelector::ConnectingResult(CUpDownClient* sender, bool success)
{
	CChatItem		*pChatItem = GetItemByClient(sender);

	if (pChatItem == NULL || pChatItem->m_pClient == NULL)
		return;

	pChatItem->m_pClient->SetChatState(MS_CHATTING);

	if (!success)
	{
		if (!pChatItem->m_strMessagePending.IsEmpty())
		{
			pChatItem->m_pLog->AppendText(_T(' ') + GetResString(IDS_FAILED) + _T('\n'), RGB(255, 0, 0));
			pChatItem->m_strMessagePending.Empty();
		}
#ifdef _DEBUG
		else
		{
			pChatItem->m_pLog->AppendText(GetResString(IDS_CHATDISCONNECTED) + _T('\n'), RGB(255, 0, 0));
		}
#endif
	}
	else if (!pChatItem->m_strMessagePending.IsEmpty())
	{
		pChatItem->m_pLog->AppendText(_T(" ok\n"), RGB(255, 0, 0));
		SendMessagePacket(*pChatItem, pChatItem->m_strMessagePending);

		COleDateTime	timelog(COleDateTime::GetCurrentTime());
		CString			strTmp;

		strTmp.Format(_T("%s (%s): "), g_eMuleApp.m_pGlobPrefs->GetUserNick(), timelog.Format());
		pChatItem->m_pLog->AppendText(strTmp, RGB(1, 180, 20));
		pChatItem->m_pLog->AppendText(pChatItem->m_strMessagePending + _T('\n'));
		pChatItem->m_strMessagePending.Empty();
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CChatSelector::DeleteAllItems()
{
	TCITEM		tcitem;

	for (int i = 0; i < GetItemCount(); i++)
	{
		tcitem.mask = TCIF_PARAM;
		if (GetItem(i, &tcitem))
			delete (CChatItem*)tcitem.lParam;
	}
	chatout.ShowWindow(SW_SHOW);
	m_pCloseBtn->EnableWindow(false);
	m_pSendBtn->EnableWindow(false);
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CChatSelector::OnTimer(UINT_PTR nIDEvent)
{
	blinkstate = !blinkstate;

	bool		globalnotify = false;
	TCITEM		tcitem;

	for (int i = 0; i < GetItemCount(); i++)
	{
		tcitem.mask = TCIF_PARAM;
		if (!GetItem(i, &tcitem))
			break;

		tcitem.mask = TCIF_IMAGE;
		if (((CChatItem*)tcitem.lParam)->m_bNotify)
		{
			tcitem.iImage = (blinkstate) ? 1 : 2;
			SetItem(i, &tcitem);
			globalnotify = true;
		}
		else if (tcitem.iImage != 0)
		{
			tcitem.iImage = 0;
			SetItem(i, &tcitem);
		}
	}
	if (globalnotify)
	{
		g_eMuleApp.m_pdlgEmule->ShowMessageState(((blinkstate) ? 1 : 2));
		lastemptyicon = false;
	}
	else if (!lastemptyicon)
	{
		g_eMuleApp.m_pdlgEmule->ShowMessageState(0);
		lastemptyicon = true;
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CChatItem* CChatSelector::GetCurrentChatItem()
{
	int			iCurSel = GetCurSel();

	if (iCurSel == -1)
		return NULL;

	TCITEM		tcitem;

	tcitem.mask = TCIF_PARAM;
	if (!GetItem(iCurSel, &tcitem))
		return NULL;
	return (CChatItem*)tcitem.lParam;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CChatSelector::ShowChat()
{
	int				iCurSel = GetCurSel();

	if (iCurSel == -1)
		return;

	bool			bWasChanged = (GetItemState(iCurSel, TCIS_HIGHLIGHTED) & TCIS_HIGHLIGHTED);

	SetItemState(iCurSel, TCIS_HIGHLIGHTED, NULL);

	CChatItem		*pChatItem = GetCurrentChatItem(), *pUpdateItem;

	if (pChatItem == NULL)
		return;

	pChatItem->m_pLog->SetRedraw(false);
	pChatItem->m_pLog->ShowWindow(SW_SHOW);
	if (pChatItem->m_pLog->m_bAutoScroll && bWasChanged)
		pChatItem->m_pLog->ScrollToLastLine();
	pChatItem->m_pLog->SetRedraw(true);

	TCITEM			tcitem;
	int				i = 0;

	tcitem.mask = TCIF_PARAM;
	while (GetItem(i++, &tcitem))
	{
		pUpdateItem = (CChatItem*)tcitem.lParam;
		if (pUpdateItem != pChatItem)
			pUpdateItem->m_pLog->ShowWindow(SW_HIDE);
	}

	pChatItem->m_pLog->SetTitle(pChatItem->m_pClient->GetUserName());
	pChatItem->m_bNotify = false;
	m_pMessageBox->SetFocus();
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CChatSelector::OnTcnSelchangeChatsel(NMHDR *pNMHDR, LRESULT *pResult)
{
	ShowChat();
	*pResult = 0;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
INT	CChatSelector::InsertItem(int nItem, TCITEM* pTabCtrlItem)
{
	if (GetItemCount() == 0)
		chatout.ShowWindow(SW_HIDE);

	int		iResult = CClosableTabCtrl::InsertItem(nItem, pTabCtrlItem);

	RedrawWindow();
	return iResult;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
BOOL CChatSelector::DeleteItem(int nItem)
{
	CClosableTabCtrl::DeleteItem(nItem);
	if (GetItemCount() == 0)
	{
		chatout.ShowWindow(SW_SHOW);
		m_pCloseBtn->EnableWindow(false);
		m_pSendBtn->EnableWindow(false);
	}
	RedrawWindow();
	return true;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CChatSelector::EndSession(CUpDownClient* client)
{
	int		iCurSel;

	if (client)
		iCurSel = GetTabByClient(client);
	else
		iCurSel = GetCurSel();

	if (iCurSel == -1)
		return;

	TCITEM		tcitem;
	
	tcitem.mask = TCIF_PARAM;

	if (!GetItem(iCurSel, &tcitem) || tcitem.lParam == 0)
		return;

	CChatItem		*pChatItem = (CChatItem*)tcitem.lParam;

	pChatItem->m_pClient->SetChatState(MS_NONE);

	DeleteItem(iCurSel);
	delete pChatItem;

	int			iTabItems = GetItemCount();

	if (iTabItems > 0)
	{
		// select next tab
		if (iCurSel == CB_ERR)
			iCurSel = 0;
		else if (iCurSel >= iTabItems)
			iCurSel = iTabItems - 1;
		(void)SetCurSel(iCurSel);				// returns CB_ERR if error or no prev. selection(!)
		iCurSel = GetCurSel();					// get the real current selection
		if (iCurSel == CB_ERR)					// if still error
			iCurSel = SetCurSel(0);
		ShowChat();
	}

}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CChatSelector::OnSize(UINT nType, int cx, int cy)
{
	CClosableTabCtrl::OnSize(nType, cx, cy);

	CRect		 rect, rClose, rSend, rMessage;

	GetClientRect(&rect);
	AdjustRect(FALSE, rect);

	m_pCloseBtn->GetWindowRect(&rClose);
	m_pCloseBtn->SetWindowPos( NULL, rect.right - 7 - rClose.Width(), rect.bottom - 7 - rClose.Height(),
	                           rClose.Width(), rClose.Height(), SWP_NOZORDER );
	m_pSendBtn->GetWindowRect(&rSend);
	m_pSendBtn->SetWindowPos( NULL, rect.right - 7 - rClose.Width() - 7 - rSend.Width(), rect.bottom - 7 - rSend.Height(),
	                          rSend.Width(), rSend.Height(), SWP_NOZORDER );

	m_pMessageBox->GetWindowRect(&rMessage);
	m_pMessageBox->SetWindowPos( NULL, rect.left + 7, rect.bottom - 9 - rMessage.Height(),
	                             rect.right - 7 - rClose.Width() - 7 - rSend.Width() - 21,
	                             rMessage.Height(), SWP_NOZORDER );

	int			iTop = rClose.Height() > rSend.Height() ? rClose.Height() : rSend.Height();

	if (iTop < rMessage.Height())
		iTop = rMessage.Height();

	if (GetItemCount() > 0)
		rect.top -= 19;

	chatout.SetWindowPos(NULL, rect.left + 7, rect.top + 7, rect.right - 18, rect.Height() - 7 - iTop - 14, SWP_NOZORDER);

	TCITEM			tcitem;
	CChatItem		*pChatItem;
	int				i = 0;

	tcitem.mask = TCIF_PARAM;
	while (GetItem(i++, &tcitem))
	{
		pChatItem = (CChatItem*)tcitem.lParam;
		pChatItem->m_pLog->SetWindowPos(NULL, rect.left + 7, rect.top + 7 + 19, rect.right - 18, rect.Height() - 7 - iTop - 14, SWP_NOZORDER);
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CChatSelector::OnBnClickedCclose()
{
	EndSession();
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CChatSelector::OnBnClickedCsend()
{
	EMULE_TRY

	CString			strNewMessage;

	m_pMessageBox->GetWindowText(strNewMessage);
	strNewMessage.Trim();
	if (!strNewMessage.IsEmpty() && SendMessage(strNewMessage))
		m_pMessageBox->SetWindowText(_T(""));
	m_pMessageBox->SetFocus();

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
BOOL CChatSelector::PreTranslateMessage(MSG* pMsg)
{
	if (pMsg->message == WM_KEYDOWN)
	{

		if (pMsg->wParam == VK_RETURN)
		{
			if (pMsg->hwnd == m_pMessageBox->m_hWnd)
				OnBnClickedCsend();
		}

		if ( (pMsg->hwnd == GetDlgItem(IDC_CMESSAGE)->m_hWnd) &&
		     (pMsg->wParam == VK_UP || pMsg->wParam == VK_DOWN) )
		{
			g_eMuleApp.m_pdlgEmule->m_wndChat.ScrollHistory(pMsg->wParam == VK_DOWN);
			return TRUE;
		}
	}
	return CClosableTabCtrl::PreTranslateMessage(pMsg);
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CChatSelector::Localize(void)
{
	if (m_hWnd)
	{
		if (m_pSendBtn)
			m_pSendBtn->SetWindowText(GetResString(IDS_CW_SEND));
		else
			GetParent()->SetDlgItemText(IDC_CSEND, GetResString(IDS_CW_SEND));
		if (m_pCloseBtn)
			m_pCloseBtn->SetWindowText(GetResString(IDS_CW_CLOSE));
		else
			GetParent()->SetDlgItemText(IDC_CCLOSE, GetResString(IDS_CW_CLOSE));
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CChatSelector::UpdateFont()
{
	chatout.SetFont(&g_eMuleApp.m_pdlgEmule->m_fontDefault);

	TCITEM		tcitem;
	CChatItem	*pChatItem;
	int			i = 0;
	
	tcitem.mask = TCIF_PARAM;
	while (GetItem(i++, &tcitem))
	{
		pChatItem = (CChatItem*)tcitem.lParam;
		pChatItem->m_pLog->SetFont(&g_eMuleApp.m_pdlgEmule->m_fontDefault);
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CChatSelector::OnEnLinkMessageBox(NMHDR *pNMHDR, LRESULT *pResult)
{
	*pResult = 0;

	ENLINK		*pEnLink = reinterpret_cast<ENLINK *>(pNMHDR);

	if (pEnLink && pEnLink->msg == WM_LBUTTONDOWN)
	{
		CString			strUrl;

		chatout.GetTextRange(pEnLink->chrg.cpMin, pEnLink->chrg.cpMax, strUrl);
		CString sLangUrl = strUrl; // GetTextRange returns '?' for non-english chars :(
		sLangUrl.Remove('?'); sLangUrl.Remove(' ');
		if ((strUrl == CLIENT_NAME) || sLangUrl.IsEmpty())
			ShellExecute(NULL, NULL, _T("http://emuleplus.info"), NULL, NULL, SW_SHOWDEFAULT);
		*pResult = 1;
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CChatSelector::OnDestroy()
{
	if (m_Timer)
		KillTimer(m_Timer);
	CClosableTabCtrl::OnDestroy();
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CChatSelector::SendAwayMessage(CChatItem* ci)
{
	if (ci == NULL  || ci->m_pClient == NULL || ci->m_pClient->GetChatState() == MS_CONNECTING) //send AwayMessage only if connected
		return false;

#ifdef OLD_SOCKETS_ENABLED
	if (ci->m_pClient->m_pRequestSocket && ci->m_pClient->m_pRequestSocket->IsConnected())
	{
		CString		strAwayMessage = _T("[AUTO REPLY:AWAY] ") + g_eMuleApp.m_pGlobPrefs->GetAwayStateMessage();
		uint16		uMessageLen = _tcslen(static_cast<LPCTSTR>(strAwayMessage));
		Packet		*pPacket = new Packet(OP_MESSAGE, uMessageLen + 2);

		memcpy2(pPacket->m_pcBuffer, &uMessageLen, 2);								// <messagelen 2>
		memcpy2(pPacket->m_pcBuffer + 2, strAwayMessage, uMessageLen);				// (<messagechar 1>)*messagelen
		g_eMuleApp.m_pUploadQueue->AddUpDataOverheadOther(pPacket->m_dwSize);
		ci->m_pClient->m_pRequestSocket->SendPacket(pPacket, true, true);

		COleDateTime		timelog(COleDateTime::GetCurrentTime());
		CString				strTmp;

		strTmp.Format(_T("%s (%s): "), g_eMuleApp.m_pGlobPrefs->GetUserNick(), timelog.Format());
		ci->m_pLog->AppendText(strTmp, RGB(1, 180, 20));
		ci->m_pLog->AppendText(strAwayMessage + _T('\n'));
	}
#endif //OLD_SOCKETS_ENABLED
	return true;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CChatSelector::SendMessagePacket(CChatItem &ci, const CString &strMessage)
{
	uint16		uMessageLen = strMessage.GetLength();
	Packet		*pPacket = new Packet(OP_MESSAGE, uMessageLen + 2);

	memcpy2(pPacket->m_pcBuffer, &uMessageLen, 2);							//<messagelen 2>
	memcpy2(pPacket->m_pcBuffer + 2, strMessage.GetString(), uMessageLen);	//(<messagechar 1>)*messagelen
	g_eMuleApp.m_pUploadQueue->AddUpDataOverheadOther(pPacket->m_dwSize);
#ifdef OLD_SOCKETS_ENABLED
	ci.m_pClient->m_pRequestSocket->SendPacket(pPacket, true, true);
#endif //OLD_SOCKETS_ENABLED
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
