// EmEngine.cpp: implementation of the CEmEngine class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#include "Other/EmWinNT.h"
#include "Other/EmMt.h"
#include "Other/ThreadPool.h"
#include "TaskProcessor.h"
#include "EmEngine.h"
#include "Sockets/TasksSockets.h"

#include "Data/Server.h"
#include "Files/KnownFile.h"
//#include "Other/ed2k_filetype.h" included indirect over KnownFile.h

#include "XML/XML.h"

//////////////////////////////////////////////////////////////////////
// Constructor
CEmEngine::CEmEngine() :
	m_hStop(NULL),
	m_stMainTP(*this),
	m_stSocketsTP(*this),
	m_stTcp(*this),
	m_stLogger(*this),
	m_hWndUI(NULL)
{
	m_stServer.nConnState = SERVER_DISCONNECTED;
	m_stServer.hSocket = NULL;
	m_stServer.nAddr = 0;
	m_stServer.nPort = 0;
	m_stServer.nClientID = 0;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// start
bool CEmEngine::Init(HWND hWndUI)
{
	if(!InitializeDatabases())
		return false;

/*	CServer stSrv;
	stSrv.ImportFromServerMet(*this, _T("./Db/server.met"));

	if(stSrv.ListStartByStatic(*this))
	{
		while(stSrv.ListGetNext())
		{
			int a = 1;
		}
		stSrv.ListFinish();
	}*/

	m_hWndUI = hWndUI;
//	Create an Event which can be asserted to stop the Engine. If successful...
	if (m_hStop = CreateEvent(NULL, TRUE, FALSE, NULL))
	{
	//	Try to initialize the Logger. If we fail...
		if (!m_stLogger.Init())
		{
			ASSERT(FALSE);
			AddDebugLogLine(false, "Failed to initialize logger engine");
			return false;
		}
	//	Initialize the Sockets TaskProcessor
		if (m_stSocketsTP.Init())
		{
		//	Initialize the TCP "Engine".		// MOREVIT - Too many engines in this thing.
			if (m_stTcp.Init())
			{
				// add relevant interfaces
				//m_stTcp.AddInterface(80, T_CLIENT_WEB);
				//m_stTcp.AddInterface(7821, T_CLIENT_PEER);
			}
			else
			{
				ASSERT(FALSE);
				AddDebugLogLine(false, "Failed to initialize tcp engine");
				return false;
			}
		}
		else
		{
			ASSERT(FALSE);
			AddDebugLogLine(false, "Failed to initialize sockets processor");
			return false;
		}
		if (!m_stMainTP.Init())
		{
			ASSERT(FALSE);
			AddDebugLogLine(false, "Failed to initialize main processor");
			return false;
		}
		//initial sort file types array
		if (!InitFileTypeArray())
		{
			ASSERT(FALSE);
			AddDebugLogLine(false, "Failed to initialize filetype array");
			return false;
		}
		return true;
	}
	else
	{
		ASSERT(FALSE);
		AddDebugLogLine(false, "CreateEvent failed");
		return false;
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define SERVERLIST_DB _T("Servers.db")
#define FILES_DB _T("files.db")

bool CEmEngine::InitializeDatabases()
{
	try
	{
		CString sDbHome = _T("./Db"); //CString(m_pGlobPrefs->GetAppDir()) + _T("Db");

		::CreateDirectory(sDbHome, NULL);	// In case it doesn't exist

		// Create environment
		m_pDbEnv = new DbEnv(0);
		m_pDbEnv->set_cachesize(0, 256 * 1024, 0);	// 256kb cache size
		m_pDbEnv->set_alloc(&malloc,&realloc,&free);
		USES_CONVERSION;
		m_pDbEnv->open( CT2CA(sDbHome), DB_CREATE | DB_INIT_LOCK | DB_INIT_LOG
			| DB_INIT_MPOOL | DB_INIT_TXN | DB_RECOVER
			| DB_THREAD, 0 );

		m_pDbEnv->set_flags(DB_AUTO_COMMIT, 1);

		// Servers list
		m_pDbServers = new Db(m_pDbEnv, 0);
		m_pDbServers->set_pagesize(16 * 1024); // 16Kb page m_lSize
		m_pDbServers->open(NULL, SERVERLIST_DB, _T("Server-List"), DB_BTREE, DB_CREATE | DB_THREAD | DB_AUTO_COMMIT, 0);

		// Servers list - primary index by addr/port
		m_pDbServersAddr = new Db(m_pDbEnv, 0);
		m_pDbServersAddr->open(NULL, SERVERLIST_DB, _T("Server-List-Addr"), DB_BTREE, DB_CREATE | DB_THREAD | DB_AUTO_COMMIT, 0);

		m_pDbServers->associate(NULL, m_pDbServersAddr, CServer::PrimaryIndex, NULL);

		// Servers list - secondary index by 'static' field
		m_pDbServersStatic = new Db(m_pDbEnv, 0);
		m_pDbServersStatic->set_flags(DB_DUP | DB_DUPSORT);
		m_pDbServersStatic->open(NULL, SERVERLIST_DB, _T("Server-List-Static"), DB_BTREE, DB_CREATE | DB_THREAD | DB_AUTO_COMMIT, 0);

		m_pDbServers->associate(NULL, m_pDbServersStatic, CServer::SecondaryIndexStatic, NULL);

		// Servers list - secondary index by 'priority' field
		m_pDbServersPriority = new Db(m_pDbEnv, 0);
		m_pDbServersPriority->set_flags(DB_DUP | DB_DUPSORT);
		m_pDbServersPriority->open(NULL, SERVERLIST_DB, _T("Server-List-Priority"), DB_BTREE, DB_CREATE | DB_THREAD | DB_AUTO_COMMIT, 0);

		m_pDbServers->associate(NULL, m_pDbServersPriority, CServer::SecondaryIndexPriority, NULL);

		// files DB
		m_pDbFiles = new Db(m_pDbEnv, 0);
		m_pDbFiles->set_pagesize(16 * 1024); // 16Kb page m_lSize
		m_pDbFiles->open(NULL, FILES_DB, _T("FileDB"), DB_BTREE, DB_CREATE | DB_THREAD | DB_AUTO_COMMIT, 0);

		// files DB - secondary index by hash
		m_pDbFilesHash = new Db(m_pDbEnv, 0);
		m_pDbFilesHash->set_flags(DB_DUP | DB_DUPSORT);
		m_pDbFilesHash->open(NULL, FILES_DB, _T("FileDB-Hash"), DB_BTREE, DB_CREATE | DB_THREAD | DB_AUTO_COMMIT, 0);

		m_pDbFiles->associate(NULL, m_pDbFilesHash, CKnownFile::FileHashIndex, NULL);

		// part DB
		m_pDbMD4Parts = new Db(m_pDbEnv, 0);
		m_pDbMD4Parts->set_pagesize(16 * 1024); // 16Kb page m_lSize
		m_pDbMD4Parts->open(NULL, FILES_DB, _T("MD4PartDB"), DB_BTREE, DB_CREATE | DB_THREAD | DB_AUTO_COMMIT, 0);

	}
	catch (DbRunRecoveryException &dbe)
	{
		TRACE(CString(dbe.what()));
		return false;
	}
	catch (DbException &dbe)
	{
		m_pDbServers = NULL;
		m_pDbServersAddr = NULL;
		m_pDbServersStatic = NULL;
		m_pDbServersPriority = NULL;
		m_pDbEnv = NULL;
		TRACE("Problems create database objects: %s. Closing.\n", dbe.what());
		return false;
	}
	catch(...)
	{
		m_pDbServers = NULL;
		m_pDbServersAddr = NULL;
		m_pDbServersStatic = NULL;
		m_pDbServersPriority = NULL;
		m_pDbEnv = NULL;
		TRACE("Problems create database objects. Closing.\n");
		return false;
	}
	return true;
}

//////////////////////////////////////////////////////////////////////
// stop
void CEmEngine::Uninit()
{
	if (m_hStop)
	{
		if (!SetEvent(m_hStop))
		{
			ASSERT(FALSE);
			AddDebugLogLine(false, "SetEvent failed");
		}

		m_stTcp.Uninit(false);
		m_stSocketsTP.Uninit();
		m_stMainTP.Uninit();

		m_stTcp.Uninit(); // final
		m_stLogger.Uninit();

		if (!CloseHandle(m_hStop))
		{
			ASSERT(FALSE);
			AddDebugLogLine(false, "CloseHandle failed");
		}
		m_hStop = NULL;
	}
	CloseDatabases();
}

void CEmEngine::CloseDatabases()
{
	try
	{
		if(m_pDbServersPriority)
		{
			m_pDbServersPriority->close(0);
			safe_delete(m_pDbServersPriority);
		}

		if(m_pDbServersStatic)
		{
			m_pDbServersStatic->close(0);
			safe_delete(m_pDbServersStatic);
		}

		if(m_pDbServersAddr)
		{
			m_pDbServersAddr->close(0);
			safe_delete(m_pDbServersAddr);
		}

		if(m_pDbServers)
		{
			m_pDbServers->close(0);
			safe_delete(m_pDbServers);
		}

		//close file DB
		if(m_pDbFilesHash)
		{
			m_pDbFilesHash->close(0);
			safe_delete(m_pDbFilesHash);
		}

		if(m_pDbFiles)
		{
			m_pDbFiles->close(0);
			safe_delete(m_pDbFiles);
		}

		if(m_pDbEnv)
		{
			m_pDbEnv->txn_checkpoint(0,0,0);
			m_pDbEnv->close(0);
			safe_delete(m_pDbEnv);
		}

		if(m_pDbMD4Parts)
		{
			m_pDbMD4Parts->close(0);
			safe_delete(m_pDbMD4Parts);
		}

	}
	catch(DbException &dbe)
	{
		TRACE("Problems delete database objects: %s.\n", dbe.what());
	}
	catch(...)
	{
		TRACE("Problems delete database objects.\n");
	}

}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// connecting to given server addr/port
//	MOREVIT: Port is LONG why?
void CEmEngine::ConnectToServer(CString sAddr/* = "" */, ULONG ulPort/* = 0 */)
{
	EMULE_TRY

	if (GetServerSocket())
	{
		DisconnectFromServer(GetServerSocket());
	}

//	AddLogLine(false, IDS_CONNECTINGTO, pServer->GetListName(), pServer->GetAddress(), pServer->GetPort());
	TRACE(_T("Connecting to server (%s:%d)\n"), sAddr, ulPort);

	CTask_LoginToServer	*pCompletionTask = new CTask_LoginToServer();
	CTask_Connect		*pTask = new CTask_Connect(sAddr, (USHORT) ulPort, T_CLIENT_SERVER, pCompletionTask);

	SetConnectionState(SERVER_CONNECTING);
	m_stSocketsTP.Push(pTask);

	EMULE_CATCH2
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// connect to any server
void CEmEngine::ConnectToAnyServer()
{
	ConnectToServer();
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// disconnect from connected server
void CEmEngine::DisconnectFromServer(SOCKET hPrevServer /* = NULL */)
{
	if(!hPrevServer)
	{
		if(GetServerSocket())
			DisconnectFromServer(GetServerSocket());
		else
			SetConnectionState(SERVER_DISCONNECTED);
		return;
	}

	CEmClient* pServer = m_stSocketsTP.Lookup(hPrevServer);
	if(!pServer)
	{
		ASSERT(FALSE);
		AddDebugLogLine(false, _T("lookup for server failed"));
		return;
	}
	CTask_KillClient* pTask = new CTask_KillClient(pServer);
	m_stSocketsTP.Push(pTask);
}

//////////////////////////////////////////////////////////////////////
// send list of shared files
void CEmEngine::SendSharedListToServer()
{
	EMULE_TRY

	CTask_SendSharedList	   *pTask = new CTask_SendSharedList();

	m_stSocketsTP.Push(pTask);

	EMULE_CATCH2
}

//////////////////////////////////////////////////////////////////////
bool CEmEngine::IsConnecting()
{
	return ( m_stServer.nConnState == SERVER_CONNECTING
		  || m_stServer.nConnState == SERVER_WAITFORLOGIN ); 
}

//////////////////////////////////////////////////////////////////////
bool CEmEngine::IsConnected()
{ 
	return (m_stServer.nConnState == SERVER_CONNECTED);
}

//////////////////////////////////////////////////////////////////////
bool CEmEngine::IsLowID()
{ 
	return (m_stServer.nClientID < 0x1000000); 
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CEmEngine::SetConnectionState(short nConnectionState)
{
	m_stServer.nConnState = nConnectionState;

//	Export connection state
	switch (nConnectionState)
	{
		case SERVER_DISCONNECTED:
		{
			m_stServer.hSocket = NULL;
			m_stServer.nAddr = 0;
			m_stServer.nPort = 0;
			m_stServer.nClientID = 0;
			TRACE("CONNECTION STATE CHANGE: DISCONNECTED\n");
			break;
		}
		case SERVER_CONNECTING:
		{
			TRACE("CONNECTION STATE CHANGE: CONNECTING\n");
			break;
		}
		case SERVER_WAITFORLOGIN:
		{
			TRACE("CONNECTION STATE CHANGE: WAITING FOR LOGIN\n");
			break;
		}
		case SERVER_CONNECTED:
		{
			TRACE("CONNECTION STATE CHANGE: CONNECTED\n");
			break;
		}
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CEmEngine::SetClientID(long cID)
{
	m_stServer.nClientID = cID; 
}

//////////////////////////////////////////////////////////////////////
void CEmEngine::ConnectedTo(CEmClient_Server* pClient)
{
	m_stServer.hSocket = pClient->m_hSocket;
	m_stServer.nAddr = pClient->m_dwAddr;
	m_stServer.nPort = pClient->m_uPort;
}

//////////////////////////////////////////////////////////////////////
bool CEmEngine::IsLocalServer(ULONG nAddr, USHORT uPort)
{
	if(!IsConnected())
		return false;
	/*	if(m_stConnected.pServer)
	{
	if (m_stConnected.pServer->GetIP() == nAddr &&
	m_stConnected.pServer->GetPort() == uPort)
	return true;
	}*/
	return false;
}

//////////////////////////////////////////////////////////////////////

#ifdef _DEBUG
static BYTE g_arrUserHash[16] = { 0x6c, 0x7c, 0x5d, 0x3b, 0x90, 0x0e, 0x11, 0x3f, 0xb1, 0x5c, 0x1c, 0x2e, 0xe8, 0x19, 0x6f, 0x59 };
#endif //_DEBUG

BYTE* CEmEngine::GetUserHash()
{
	//	return g_eMuleApp.m_pGlobPrefs->GetUserHash();
#ifdef _DEBUG
	return g_arrUserHash;
#else
	return NULL;
#endif
}

USHORT CEmEngine::GetPort()
{
	//	return g_eMuleApp.m_pGlobPrefs->GetPort();
	return 4662;
}

CString CEmEngine::GetUserNick()
{
	//	return g_eMuleApp.m_pGlobPrefs->GetUserNick();
	return _T("ePlus v2 debug (rus)(de)");
}

/*
// Contain all information about connecting/connected client
class CEmClientData
{
public:
	CPartFile*	GetFile() const { return m_pPartFile; }

private:
	CPartFile	*m_pPartFile;
};


//////////////////////////////////////////////////////////////////////
// Executed in GUI thread
void CEngineData::ImportServerState(CTask_ServerState* pTask)
{
	if(!pTask)
	{
		ASSERT(FALSE);
		return;
	}

	// should keep independent serverlist in the future
	if(!m_pServerList)
		m_pServerList = g_eMuleApp.m_pServerList;

	if(m_nClientID != pTask->m_nClientID)
	{
		m_nClientID = pTask->m_nClientID;
		AddLogLine(false, IDS_NEWCLIENTID, m_nClientID);
	}

	pState->pServer = m_pServerList->GetServerByIP(pState->nAddr, pState->nPort);

	if(pState->nConnState != pTask->m_stState.nConnState)
	{
		pState->nConnState = pTask->m_stState.nConnState;
		switch(pState->nConnState)
		{
		case SERVER_DISCONNECTED:
			if(m_stConnected.nConnState == SERVER_DISCONNECTED)
				g_eMuleApp.m_pdlgEmule->ShowConnectionState(false);
			break;
		case SERVER_CONNECTING:
			g_eMuleApp.m_pdlgEmule->ShowConnectionState(false);
			break;
		case SERVER_WAITFORLOGIN:
			if(m_stConnecting.pServer)
				AddLogLine(false, IDS_CONNECTEDTOREQ,
				m_stConnecting.pServer->GetListName(),
				m_stConnecting.pServer->GetFullIP(),
				m_stConnecting.pServer->GetPort());
			break;
		case SERVER_CONNECTED:
			if(m_stConnected.pServer)
			{
				AddLogLine(true, IDS_CONNECTEDTO, m_stConnected.pServer->GetListName());
				g_eMuleApp.m_pdlgEmule->ShowConnectionState(true, m_stConnected.pServer->GetListName());
				//				if(IsLowID())
				//					AddLogLine(false, GetResString(IDS_GOTLOWID));

				// Write online signature
				g_eMuleApp.OnlineSig();

				// Resume downloading files
				g_eMuleApp.m_pDownloadQueue->ResumeFiles();

				m_stEngine.SendSharedListToServer();
			}
			break;
		}
	}
}
*/
