#include "stdafx.h"
#include "eMule.h"
#include "HTRichEditCtrl.h"
#include "otherfunctions.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

static const struct
{
	LPCTSTR pszScheme;
} _apszSchemes[] =
{
	{ _T("ed2k://") },
	{ _T("ftp.") },
	{ _T("ftp://") },
	{ _T("http://") },
	{ _T("https://") },
	{ _T("mailto:") },
	{ _T("www.") },
	{ _T("www1.") },
	{ _T("www2.") },
	{ _T("www3.") },
	{ _T("www4.") },
	{ _T("www5.") }
};

IMPLEMENT_DYNAMIC(CHTRichEditCtrl, CRichEditCtrl)

BEGIN_MESSAGE_MAP(CHTRichEditCtrl, CRichEditCtrl)
	ON_WM_CONTEXTMENU()
	ON_WM_SETCURSOR()
	ON_WM_KEYDOWN()
	ON_WM_PAINT()
	ON_NOTIFY_REFLECT_EX(EN_LINK, OnEnLink)
	ON_WM_CREATE()
END_MESSAGE_MAP()

CHTRichEditCtrl::CHTRichEditCtrl()
{
	m_bAutoScroll = true;
	m_bWordWrap = false;
	m_bNoPaint = false;
	m_bInMenu = false;
	m_strTitle.Empty();
	switch (get_cpu_type())
	{
	case 1:
		m_lMaxBufSize = (32*1024);
		break;
	case 2:
		m_lMaxBufSize = (64*1024);
		break;
	default:
		if (g_eMuleApp.m_pGlobPrefs->GetWindowsVersion() == _WINVER_95_)
			m_lMaxBufSize = (64*1024);
		else
			m_lMaxBufSize = (128*1024);
		break;
	}
	m_hCursor = ::LoadCursor(NULL, MAKEINTRESOURCE(32512));
}

CHTRichEditCtrl::~CHTRichEditCtrl()
{
	if (m_hCursor)
		::DestroyCursor(m_hCursor);
}

void CHTRichEditCtrl::SetTitle(LPCTSTR pszTitle)
{
	m_strTitle = pszTitle;
}

void CHTRichEditCtrl::Init()
{
	VERIFY( SendMessage(EM_SETUNDOLIMIT, 0, 0) == 0 );
	if (g_eMuleApp.m_pGlobPrefs->GetWindowsVersion() == _WINVER_95_)
		LimitText((64*1024)-1);
	else
		LimitText(128*1024);
	
	SetTargetDevice(NULL, !m_bWordWrap);
}

int CHTRichEditCtrl::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CRichEditCtrl::OnCreate(lpCreateStruct) == -1)
		return -1;
	Init();
	return 0;
}

LRESULT CHTRichEditCtrl::WindowProc(UINT message, WPARAM wParam, LPARAM lParam)
{
	if ((message == WM_ERASEBKGND || message == WM_PAINT) && m_bNoPaint)
		return true;

	return CRichEditCtrl::WindowProc(message, wParam, lParam);
}

void CHTRichEditCtrl::AddLine(LPCTSTR pszMsg, COLORREF crTextColor /*= 0x1000000*/, COLORREF crBackColor /*= 0x1000000*/, DWORD dwFlags /*= 0*/)
{
	long		lMsgLen = _tcslen(pszMsg);

	if (lMsgLen == 0)
		return;

	long		lStartChar, lEndChar, lTextEnd = GetTextLengthEx(GTL_PRECISE);

	GetSel(lStartChar, lEndChar);

	bool			bIsVisible = IsWindowVisible(), bIsEndSel = (lStartChar == lEndChar && lStartChar == lTextEnd);
	CHARFORMAT2		cf2;
	POINT			ptOldPos;

	if (bIsVisible)
	{
		m_bNoPaint = true;
		SetRedraw(false);
	}

	if (lTextEnd >= m_lMaxBufSize)
	{
		CString		strLog;

		GetWindowText(strLog);

		long		lLen = strLog.Find(_T('\n'), m_lMaxBufSize >> 3);

		if (lLen == -1)
			lLen = m_lMaxBufSize >> 3;

		strLog.Truncate(lLen);
		strLog.Remove(_T('\n'));
		lLen = strLog.GetLength();
		SetSel(0, lLen);
		ReplaceSel(_T(""));
		lTextEnd = GetTextLengthEx(GTL_PRECISE);
	}

	SendMessage(EM_GETSCROLLPOS, 0, (LPARAM)&ptOldPos);
	if (!bIsEndSel)
		SetSel(lTextEnd, lTextEnd);

	memzero(&cf2, sizeof cf2);
	cf2.cbSize = sizeof cf2;
	GetSelectionCharFormat(cf2);
	cf2.dwEffects |= (CFE_AUTOCOLOR | CFE_AUTOBACKCOLOR);
	cf2.dwEffects &= ~(CFE_BOLD | CFE_ITALIC | CFE_LINK | CFE_STRIKEOUT | CFE_UNDERLINE);
	if ((crTextColor & 0x1000000) == 0)
	{
		cf2.dwEffects &= ~CFE_AUTOCOLOR;
		cf2.crTextColor = (crTextColor & 0xFFFFFF);
	}
	if ((crBackColor & 0x1000000) == 0)
	{
		cf2.dwEffects &= ~CFE_AUTOBACKCOLOR;
		cf2.crBackColor = (crBackColor & 0xFFFFFF);
	}
	if ((dwFlags & HTC_BOLD) != 0)
		cf2.dwEffects |= CFE_BOLD;
	if ((dwFlags & HTC_ITALIC) != 0)
		cf2.dwEffects |= CFE_ITALIC;
	if ((dwFlags & HTC_LINK) != 0)
		cf2.dwEffects |= CFE_LINK;
	if ((dwFlags & HTC_STRIKEOUT) != 0)
		cf2.dwEffects |= CFE_STRIKEOUT;
	if ((dwFlags & HTC_UNDERLINE) != 0)
		cf2.dwEffects |= CFE_UNDERLINE;
	SetSelectionCharFormat(cf2);
	ReplaceSel(pszMsg);

	if (bIsEndSel)
	{
		lTextEnd = GetTextLengthEx(GTL_PRECISE);
		SetSel(lTextEnd, lTextEnd);
	}
	else
		SetSel(lStartChar, lEndChar);

	if (m_bAutoScroll)
	{
		ScrollToLastLine();

		POINT			ptNewPos;

		SendMessage(EM_GETSCROLLPOS, 0, (LPARAM)&ptNewPos);
		ptOldPos.y = ptNewPos.y;
	}
	SendMessage(EM_SETSCROLLPOS, 0, (LPARAM)&ptOldPos);

	if (bIsVisible)
	{
		m_bNoPaint = false;
		SetRedraw();
		Invalidate();
	}
}

void CHTRichEditCtrl::ScrollToLastLine()
{
	long		lFirstVisible = GetFirstVisibleLine();

	if (lFirstVisible > 0)
		LineScroll(-lFirstVisible, 0);

	// WM_VSCROLL does not work correctly under Win98 (or older versions of comctl.dll)
	SendMessage(WM_VSCROLL, SB_BOTTOM);
	SendMessage(WM_VSCROLL, SB_LINEUP);
	if (g_eMuleApp.m_pGlobPrefs->GetWindowsVersion() == _WINVER_95_)
	{
		// older versions of comctl.dll seem to need this to properly update the display
		SendMessage(WM_VSCROLL, MAKELONG(SB_THUMBPOSITION, GetScrollPos(SB_VERT)));
		SendMessage(WM_VSCROLL, SB_ENDSCROLL);
	}
}

void CHTRichEditCtrl::Reset()
{
	SetRedraw(false);
	SetWindowText(_T(""));
	SetRedraw();
	Invalidate();
}

void CHTRichEditCtrl::OnContextMenu(CWnd *pWnd, CPoint point)
{
	long			lSelStart, lSelEnd;
	CTitleMenu		menuLog;

	GetSel(lSelStart, lSelEnd);
	m_bInMenu = true;
	menuLog.CreatePopupMenu();
	menuLog.AddMenuTitle(GetResString(IDS_LOGENTRY));
	menuLog.AppendMenu(MF_STRING | ((lSelEnd > lSelStart) ? MF_ENABLED : MF_GRAYED), MP_COPYSELECTED, GetResString(IDS_COPY));

	UINT			dwFlags = MF_STRING | ((GetWindowTextLength() > 0) ? MF_ENABLED : MF_GRAYED);
	CString			strBuffer = GetResString(IDS_SAVETOFILE);

	menuLog.AppendMenu(dwFlags, MP_SAVELOG, strBuffer + _T(" (.log)"));
	menuLog.AppendMenu(dwFlags, MP_SAVERTF, strBuffer + _T(" (.rtf)"));
	menuLog.AppendMenu(MF_SEPARATOR);
	menuLog.AppendMenu(dwFlags, MP_SELECTALL, GetResString(IDS_SELECTALL));
	menuLog.AppendMenu(dwFlags, MP_REMOVEALL, GetResString(IDS_PW_RESET));

	menuLog.AppendMenu(MF_SEPARATOR);
	menuLog.AppendMenu(MF_STRING | (m_bAutoScroll ? MF_CHECKED : MF_UNCHECKED), MP_AUTOSCROLL, GetResString(IDS_AUTOSCROLL));
	menuLog.AppendMenu(MF_STRING | (m_bWordWrap ? MF_CHECKED : MF_UNCHECKED), MP_WORDWRAP, GetResString(IDS_WORDWRAP));

	menuLog.TrackPopupMenuEx(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y, this, NULL);
	m_bInMenu = false;
}

BOOL CHTRichEditCtrl::OnCommand(WPARAM wParam, LPARAM lParam)
{
	switch (wParam)
	{
	case MP_COPYSELECTED:
		Copy();
		break;
	case MP_SAVELOG:
		SaveLogToDisk();
		break;
	case MP_SAVERTF:
		SaveRtfToDisk();
		break;
	case MP_SELECTALL:
		SetSel(0, -1);
		break;
	case MP_REMOVEALL:
		Reset();
		break;
	case MP_AUTOSCROLL:
		m_bAutoScroll = !m_bAutoScroll;
		break;
	case MP_WORDWRAP:
		m_bWordWrap = !m_bWordWrap;
		SetTargetDevice(NULL, !m_bWordWrap);
		break;
	}
	return TRUE;
}

BOOL CHTRichEditCtrl::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message)
{
	if (m_bInMenu)
	{
		::SetCursor(m_hCursor);
		return TRUE;
	}
	return CWnd::OnSetCursor(pWnd, nHitTest, message);
}

void CHTRichEditCtrl::SaveLogToDisk()
{
	CString			strLog, strTemp;

	if (DialogBrowseFile(strTemp, _T("Log (*.log)|*.log||"), m_strTitle + _T(".log"), 0, false))
	{
		HANDLE		hFile = CreateFile(strTemp, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_ARCHIVE, NULL);

		if (hFile != INVALID_HANDLE_VALUE)
		{
			GetWindowText(strLog);

			DWORD		dwWritten;

			WriteFile(hFile, strLog, strLog.GetLength(), &dwWritten, NULL);
			CloseHandle(hFile);
		}
	}
}

void CHTRichEditCtrl::SaveRtfToDisk()
{
	CString			strLog, strTemp;

	if (DialogBrowseFile(strTemp, _T("Rich Text Format (RTF)|*.rtf||"), m_strTitle + _T(".rtf"), 0, false))
	{
		HANDLE		hFile = CreateFile(strTemp, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_ARCHIVE, NULL);

		if (hFile != INVALID_HANDLE_VALUE)
		{
			strTemp.Empty();
			strLog.Empty();

			EDITSTREAM		es;

			es.dwCookie = (DWORD)&strTemp;
			es.pfnCallback = MEditStreamOutCallback;
			StreamOut(SF_RTF, es);

			LPCTSTR			psz = strTemp;
			LPCTSTR			pszStart = psz;
			long			lLen;
			bool			bFoundScheme;

			while (*psz != _T('\0'))
			{
				bFoundScheme = false;

				for (int i = 0; i < ARRSIZE(_apszSchemes); i++)
				{
					if (CString(psz, _tcslen(_apszSchemes[i].pszScheme)).MakeLower() == CString(_apszSchemes[i].pszScheme))
					{
						if (psz - pszStart > 0)
							strLog += CString(pszStart, psz - pszStart);
						lLen = _tcscspn(psz, _T(" \n\r\t\\"));
						while ((lLen > 0) && !((*(psz + lLen - 1) >= _T('/') && *(psz + lLen - 1) <= _T('9')) ||
							(*(psz + lLen - 1) >= _T('A') && *(psz + lLen - 1) <= _T('Z')) ||
							(*(psz + lLen - 1) >= _T('a') && *(psz + lLen - 1) <= _T('z')) ||
							(*(psz + lLen - 1) == _T('|'))))
							lLen --;
						if (lLen == 0)
						{
							strLog += _T("{\\field{\\*\\fldinst {HYPERLINK \"") + CString(psz) + _T("\"}}{\\fldrslt{{") + CString(psz) + _T("}}}}");
							psz += _tcslen(psz);
						}
						else
						{
							strLog += _T("{\\field{\\*\\fldinst {HYPERLINK \"") + CString(psz, lLen) + _T("\"}}{\\fldrslt{{") + CString(psz, lLen) + _T("}}}}");
							psz += lLen;
						}
						pszStart = psz;
						bFoundScheme = true;
						break;
					}
				}
				if (!bFoundScheme)
					psz = _tcsinc(psz);
			}

			if (pszStart != psz)
				strLog += pszStart;

			DWORD		dwWritten;

			WriteFile(hFile, strLog, strLog.GetLength(), &dwWritten, NULL);
			CloseHandle(hFile);
		}
	}
}

void CHTRichEditCtrl::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	if (nChar == 'A' && (GetKeyState(VK_CONTROL) & 0x8000))
	{
		//////////////////////////////////////////////////////////////////
		// Ctrl+A: Select all items
		SetSel(0, -1);
	}
	else if (nChar == 'C' && (GetKeyState(VK_CONTROL) & 0x8000))
	{
		//////////////////////////////////////////////////////////////////
		// Ctrl+C: Copy listview items to clipboard
		Copy();
	}

	CRichEditCtrl::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CHTRichEditCtrl::AppendText(const CString& strText, COLORREF crTextColor /*= 0x1000000*/, COLORREF crBackColor /*= 0x1000000*/, DWORD dwFlags /*= 0*/)
{
	LPCTSTR			psz = strText;
	LPCTSTR			pszStart = psz;
	long			lLen;
	bool			bFoundScheme;

	while (*psz != _T('\0'))
	{
		bFoundScheme = false;

		for (int i = 0; i < ARRSIZE(_apszSchemes); i++)
		{
			if (CString(psz, _tcslen(_apszSchemes[i].pszScheme)).MakeLower() == CString(_apszSchemes[i].pszScheme))
			{
				if (psz - pszStart > 0)
					AddLine(CString(pszStart, psz - pszStart), crTextColor, crBackColor, dwFlags);

				lLen = _tcscspn(psz, _T(" \n\r\t"));
				while ((lLen > 0) && !((*(psz + lLen - 1) >= _T('/') && *(psz + lLen - 1) <= _T('9')) ||
						(*(psz + lLen - 1) >= _T('A') && *(psz + lLen - 1) <= _T('Z')) ||
						(*(psz + lLen - 1) >= _T('a') && *(psz + lLen - 1) <= _T('z')) ||
						(*(psz + lLen - 1) == _T('|'))))
					lLen --;
				if (lLen == 0)
				{
					AddLine(psz, crTextColor, crBackColor, (dwFlags | HTC_LINK));
					psz += _tcslen(psz);
				}
				else
				{
					AddLine(CString(psz, lLen), crTextColor, crBackColor, (dwFlags | HTC_LINK));
					psz += lLen;
				}
				pszStart = psz;
				bFoundScheme = true;
				break;
			}
		}
		if (!bFoundScheme)
			psz = _tcsinc(psz);
	}
	if (pszStart != psz)
		AddLine(pszStart, crTextColor, crBackColor, dwFlags);
}

BOOL CHTRichEditCtrl::OnEnLink(NMHDR *pNMHDR, LRESULT *pResult)
{
	bool		bMsgHandled = false;
	ENLINK		*pEnLink = reinterpret_cast<ENLINK *>(pNMHDR);

	*pResult = 0;

	if (pEnLink && pEnLink->msg == WM_LBUTTONDOWN)
	{
		CString			strUrl;

		GetTextRange(pEnLink->chrg.cpMin, pEnLink->chrg.cpMax, strUrl);

		for (int i = 0; i < ARRSIZE(_apszSchemes); i++)
		{
			if (strUrl.Left(_tcslen(_apszSchemes[i].pszScheme)).MakeLower() == CString(_apszSchemes[i].pszScheme))
			{
				ShellExecute(NULL, NULL, strUrl, NULL, NULL, SW_SHOWDEFAULT);
				*pResult = 1;
				bMsgHandled = true;
				break;
			}
		}
	}
	return bMsgHandled;
}

CString CHTRichEditCtrl::GetText() const
{
	CString			strText;

	GetWindowText(strText);
	return strText;
}

CString CHTRichEditCtrl::GetHtml()
{
	CString			strTemp, strText;
	EDITSTREAM		es;

	es.dwCookie = (DWORD)&strTemp;
	es.pfnCallback = MEditStreamOutCallback;
	StreamOut(SF_RTF, es);
	strTemp.Replace(_T("&"), _T("&amp;"));
	strTemp.Replace(_T("<"), _T("&lt;"));
	strTemp.Replace(_T(">"), _T("&gt;"));
	strTemp.Replace(_T("\""), _T("&quot;"));
	strTemp.Replace(_T("\\\'"), _T(" \\\'"));

	int				iTableStart = strTemp.Find(_T("\n{\\colortbl ;"));
	int				iTableEnd = strTemp.Find(_T(";}"), iTableStart + 1);

	if ((iTableStart != -1) && (iTableEnd != -1))
	{
		CStringArray	astrColorsRtf, astrColorsHtml;
		int				iRtfColor = 0, iColorEnd;
		long			lHtmlColor = 0;
		CString			strColor;

		while ((iTableStart < iTableEnd) && (iTableStart != -1))
		{
			iTableStart = strTemp.Find(_T("\\red"), iTableStart + 1);
			strColor.Format(_T("\\cf%i"), iRtfColor);
			astrColorsRtf.Add(strColor);
			strColor.Format(_T(" </font><font color=\"#%06x\">"), lHtmlColor);
			astrColorsHtml.Add(strColor);
			iRtfColor ++;
			if (iTableStart != -1)
			{
				iColorEnd = strTemp.Find(_T('\\'), iTableStart + 1);
				if (iColorEnd != -1)
				{
					lHtmlColor = (_tstoi(strTemp.Mid(iTableStart + 4, iColorEnd - (iTableStart + 4))) << 16);
					iTableStart = iColorEnd + 6;
					iColorEnd = strTemp.Find(_T('\\'), iTableStart + 1);
					if (iColorEnd != -1)
					{
						lHtmlColor += (_tstoi(strTemp.Mid(iTableStart, iColorEnd - iTableStart)) << 8);
						iTableStart = iColorEnd + 5;
						iColorEnd = strTemp.Find(_T(';'), iTableStart + 1);
						if (iColorEnd != -1)
							lHtmlColor += _tstoi(strTemp.Mid(iTableStart, iColorEnd - iTableStart));
					}
				}
			}
		}
		if (astrColorsRtf.GetSize() > 0)
		{
			for (int i = 0; i < astrColorsRtf.GetSize(); i++)
			{
				strTemp.Replace(astrColorsRtf[i] + _T(" \\\'"), astrColorsHtml[i] + _T(" \\\'"));
				strTemp.Replace(astrColorsRtf[i] + _T(' '), astrColorsHtml[i]);
				strTemp.Replace(astrColorsRtf[i] + _T('\\'), astrColorsHtml[i] + _T('\\'));
			}
		}

		LPCTSTR			psz = strTemp;

		psz += strTemp.Find(_T('\\'), iTableEnd);

		LPCTSTR			pszStart = psz;
		TCHAR			strChar[2];
		long			lLen;

		while (*psz != _T('\0'))
		{
			if (*psz == _T('\\'))
			{
				if (*(psz + 1) == _T('\\'))
				{
					strText += CString(pszStart, 1 + psz - pszStart);
					psz += 2;
				}
				else if (*(psz + 1) == _T('\''))
				{
					if (psz - pszStart > 0)
						strText += CString(pszStart, psz - pszStart - 1);

					strChar[0] = *(psz + 2);
					strChar[1] = *(psz + 3);
					strText += (TCHAR)_tcstol(strChar, NULL, 16);
					psz += 4;
				}
				else
				{
					if (psz - pszStart > 0)
						strText += CString(pszStart, psz - pszStart);

					lLen = _tcscspn(psz, _T(" \n"));
					if (lLen == 0)
						psz += _tcslen(psz);
					else
					{
						psz += lLen;
						if (*psz == _T(' '))
							psz++;
					}
				}
				pszStart = psz;
			}
			else
				psz++;
		}
		if (pszStart != psz)
			strText += pszStart;
	}

	iTableEnd = strText.ReverseFind(_T('}'));
	if (iTableEnd != -1)
		strText.Truncate(iTableEnd);
	strText.TrimRight('\n');
	strText.Replace(_T(" </font><font color"), _T("</font><font color"));

	return strText;
}

CString CHTRichEditCtrl::GetToolTip()
{
	CString			strTemp, strText;
	EDITSTREAM		es;

	es.dwCookie = (DWORD)&strTemp;
	es.pfnCallback = MEditStreamOutCallback;
	StreamOut(SF_RTF, es);
	strTemp.Replace(_T("<"), _T("<<"));
	strTemp.Replace(_T("\\\'"), _T(" \\\'"));

	int				iTableStart = strTemp.Find(_T("\n{\\colortbl ;"));
	int				iTableEnd = strTemp.Find(_T(";}"), iTableStart + 1);

	if ((iTableStart != -1) && (iTableEnd != -1))
	{
		CStringArray	astrColorsRtf, astrColorsTT;
		int				iRtfColor = 0, iColorEnd;
		long			lTTColor = 0;
		CString			strColor;

		while ((iTableStart < iTableEnd) && (iTableStart != -1))
		{
			iTableStart = strTemp.Find(_T("\\red"), iTableStart + 1);
			strColor.Format(_T("\\cf%i"), iRtfColor);
			astrColorsRtf.Add(strColor);
			(iRtfColor == 0 || lTTColor == GetSysColor(COLOR_WINDOWTEXT)) ? strColor = _T(" <ct>") : strColor.Format(_T(" <ct=0x%06x>"), lTTColor);
			astrColorsTT.Add(strColor);
			iRtfColor++;
			if (iTableStart != -1)
			{
				iColorEnd = strTemp.Find(_T('\\'), iTableStart + 1);
				if (iColorEnd != -1)
				{
					lTTColor = _tstoi(strTemp.Mid(iTableStart + 4, iColorEnd - (iTableStart + 4)));
					iTableStart = iColorEnd + 6;
					iColorEnd = strTemp.Find(_T('\\'), iTableStart + 1);
					if (iColorEnd != -1)
					{
						lTTColor += (_tstoi(strTemp.Mid(iTableStart, iColorEnd - iTableStart)) << 8);
						iTableStart = iColorEnd + 5;
						iColorEnd = strTemp.Find(_T(';'), iTableStart + 1);
						if (iColorEnd != -1)
							lTTColor += (_tstoi(strTemp.Mid(iTableStart, iColorEnd - iTableStart)) << 16);
					}
				}
			}
		}
		if (astrColorsRtf.GetSize() > 0)
		{
			for (int i = 0; i < astrColorsRtf.GetSize(); i++)
			{
				strTemp.Replace(astrColorsRtf[i] + _T(" \\\'"), astrColorsTT[i] + _T(" \\\'"));
				strTemp.Replace(astrColorsRtf[i] + _T(' '), astrColorsTT[i]);
				strTemp.Replace(astrColorsRtf[i] + _T('\\'), astrColorsTT[i] + _T('\\'));
			}
		}

		int			iLines = 0, iStrEnd = strTemp.ReverseFind(_T('}'));

		if (iStrEnd != -1)
			strTemp.Truncate(iStrEnd);
		strTemp.TrimRight('\n');

		LPCTSTR		pszStart = strTemp;
		LPCTSTR		psz = pszStart;

		pszStart += strTemp.Find(_T('\\'), iTableEnd);
		psz += strTemp.GetLength() - 1;

		CString		strLog, strStatus = g_eMuleApp.m_pdlgEmule->m_ctlStatusBar.GetText(SB_MESSAGETEXT, 0);
		int			iLastFound, iNextFound, iCount = 11;

		GetWindowText(strLog);
		iLastFound = strLog.Find(strStatus);
		iNextFound = iLastFound;
		while (iNextFound != -1)
		{
			iLastFound = iNextFound;
			iNextFound = strLog.Find(strStatus, iLastFound + 1);
		}
		
		if (iLastFound != -1)
		{
			iCount = 0;
			iNextFound = strLog.Find(_T("\n"), iLastFound + 1);
			while (iNextFound != -1)
			{
				iCount++;
				iNextFound = strLog.Find(_T("\n"), iNextFound + 1);
			}
			
			while (iCount > 0 && psz > pszStart)
			{
				if (*psz == _T('\n'))
					iCount--;
				psz--;
			}
			iCount = 10 - iCount;
			strTemp.Truncate(psz - strTemp);
		}

		while (iLines < iCount && psz > pszStart)
		{
			if (*psz == _T('\n'))
				iLines++;
			psz--;
		}

		if (psz > pszStart)
			psz++;

		pszStart = psz;

		TCHAR		strChar[2];
		long		lLen;

		while (*psz != _T('\0'))
		{
			if (*psz == _T('\\'))
			{
				if (*(psz + 1) == _T('\\'))
				{
					strText += CString(pszStart, 1 + psz - pszStart);
					psz += 2;
				}
				else if (*(psz + 1) == _T('\''))
				{
					if (psz - pszStart > 0)
						strText += CString(pszStart, psz - pszStart - 1);

					strChar[0] = *(psz + 2);
					strChar[1] = *(psz + 3);
					strText += (TCHAR)_tcstol(strChar, NULL, 16);
					psz += 4;
				}
				else
				{
					if (psz - pszStart > 0)
						strText += CString(pszStart, psz - pszStart);

					lLen = _tcscspn(psz, _T(" \n"));
					if (lLen == 0)
						psz += _tcslen(psz);
					else
					{
						psz += lLen;
						if (*psz == _T(' '))
							psz++;
					}
				}
				pszStart = psz;
			}
			else
				psz++;
		}
		if (pszStart != psz)
			strText += pszStart;
	}

	strText.Trim(_T('\n'));
	strText.Replace(_T(" <ct"), _T("<ct"));
	while (strText.Right(4) == _T("<ct>"))
		strText.Truncate(strText.GetLength() - 4);
	strText.TrimRight(_T('\n'));
	strText.Replace(_T("\n"), _T("<br>"));
	if (strText.GetAt(0) != _T('<'))
		strText.Insert(0, _T("<"));

	return strText;
}

void CHTRichEditCtrl::SetFont(CFont* pFont, bool bRedraw /*= true*/)
{
	LOGFONT			lf = {0};
	CHARFORMAT2		cf2;

	pFont->GetLogFont(&lf);
	memzero(&cf2, sizeof cf2);
	cf2.cbSize = sizeof cf2;
	cf2.dwMask = CFM_FACE | CFM_SIZE /*| CFM_CHARSET*/;

	if (GetWindowTextLength() == 0)
		cf2.dwMask |= CFM_BACKCOLOR | CFM_BOLD | CFM_COLOR | CFM_ITALIC | CFM_LINK | CFM_STRIKEOUT | CFM_UNDERLINE;

	cf2.crTextColor = GetSysColor(COLOR_WINDOWTEXT);
	cf2.crBackColor = GetSysColor(COLOR_WINDOW);

	HDC				hDC = ::GetDC(NULL);

	cf2.yHeight = (-MulDiv(lf.lfHeight, 72, GetDeviceCaps(hDC, LOGPIXELSY))) * 20;
	::ReleaseDC(NULL, hDC);

	cf2.bPitchAndFamily = lf.lfPitchAndFamily;
	_tcsncpy(cf2.szFaceName, lf.lfFaceName, ARRSIZE(cf2.szFaceName));
	cf2.szFaceName[ARRSIZE(cf2.szFaceName) - 1] = _T('\0');

	// although this should work correctly (according SDK) it may give false results (e.g. the "click here..." text
	// which is shown in the server info window may not be entirely used as a hyperlink???)
	//cf2.bCharSet = lf.lfCharSet;
	//cf2.yOffset = 0;
	VERIFY( SetDefaultCharFormat(cf2) );

	if (bRedraw)
	{
		Invalidate();
		UpdateWindow();
	}
}