//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

//	IrcWnd.cpp : implementation file
//

#include "stdafx.h"
#include "IrcWnd.h"
#include "emule.h"
#include "otherfunctions.h"
#include "opcodes.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#define new DEBUG_NEW
#endif

static const COLORREF _acrColorPal[16] = {
	RGB(255, 255, 255), RGB(0, 0, 0), RGB(0, 0, 127), RGB(0, 127, 0),
	RGB(255, 0, 0), RGB(127, 0, 0), RGB(156, 0, 156), RGB(252, 127, 0),
	RGB(255, 255, 0), RGB(0, 252, 0), RGB(0, 147, 147), RGB(0, 255, 255),
	RGB(0, 0, 252), RGB(255, 0, 255), RGB(127, 127, 127), RGB(210, 210, 210)
};

/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////
//	CIrcWnd dialog
/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////
IMPLEMENT_DYNAMIC(CIrcWnd, CDialog)
CIrcWnd::CIrcWnd(CWnd* pParent /*=NULL*/)
		: CResizableDialog(CIrcWnd::IDD, pParent)
{
	m_pIrcMain = NULL;
	m_bConnected = false;
	m_bLoggedIn = false;
	m_pCurrentChannel = NULL;
	memset(&m_bSortAscendingNickList, true, sizeof(m_bSortAscendingNickList));
	memset(&m_bSortAscendingChanList, true, sizeof(m_bSortAscendingChanList));
}

CIrcWnd::~CIrcWnd()
{
	if (m_bConnected)
		m_pIrcMain->Disconnect(true);

	POSITION	pos1, pos2;

	for (pos1 = channelLPtrList.GetHeadPosition(); (pos2 = pos1) != NULL;)
	{
		ChannelList* cur_channel = (ChannelList*)channelLPtrList.GetNext(pos1);
		channelLPtrList.RemoveAt(pos2);
		delete cur_channel;
	}
	DeleteAllChannel();
	delete m_pIrcMain;
}

void CIrcWnd::UpdateNickCount()
{
	CHeaderCtrl		*pHeaderCtrl = nickList.GetHeaderCtrl();
	HDITEM			hdi;
	CString			strRes;

	hdi.mask = HDI_TEXT;
	if (nickList.GetItemCount() > 0)
		strRes.Format(_T("%s[%i]"), GetResString(IDS_IRC_NICK), nickList.GetItemCount());
	else
		strRes = GetResString(IDS_IRC_NICK);
	hdi.pszText = (LPSTR)strRes.GetString();
	pHeaderCtrl->SetItem(0, &hdi);
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CIrcWnd::Localize()
{
	static const uint16 s_auResTbl[] =
	{
		IDS_IRC_NAME,
		IDS_UUSERS,
		IDS_DESCRIPTION
	};

	SetDlgItemText(IDC_BN_IRCCONNECT, GetResString((m_bConnected) ? IDS_IRC_DISCONNECT : IDS_IRC_CONNECT));
	SetDlgItemText(IDC_CHATSEND, GetResString(IDS_IRC_SEND));
	SetDlgItemText(IDC_CLOSECHAT, GetResString(IDS_FD_CLOSE));

	TCITEM		tcitem;
	Channel		*pCurChannel;

	for (int i = 0; i < channelselect.GetItemCount();i++)
	{
		tcitem.mask = TCIF_PARAM;
		tcitem.lParam = -1;
		channelselect.GetItem(i, &tcitem);
		pCurChannel = (Channel*)tcitem.lParam;
		if (pCurChannel != NULL)
		{
			if (pCurChannel->type == 1)
			{
				pCurChannel->title = GetResString(IDS_STATUS);
				tcitem.mask = TCIF_TEXT;
				tcitem.pszText = pCurChannel->title.GetBuffer();
				channelselect.SetItem(i, &tcitem);
				pCurChannel->title.ReleaseBuffer();
			}
			else if (pCurChannel->type == 2)
			{
				pCurChannel->title = GetResString(IDS_IRC_CHANNELLIST);
				tcitem.mask = TCIF_TEXT;
				tcitem.pszText = pCurChannel->title.GetBuffer();
				channelselect.SetItem(i, &tcitem);
				pCurChannel->title.ReleaseBuffer();
			}
		}
	}
	if (m_pCurrentChannel != NULL)
	{
		if (m_pCurrentChannel->type == 1)
			titleWindow.SetWindowText(GetResString(IDS_STATUS));
		else if(m_pCurrentChannel->type == 2)
			titleWindow.SetWindowText(GetResString(IDS_IRC_CHANNELLIST));
	}

	CHeaderCtrl	*pHeaderCtrl = nickList.GetHeaderCtrl();
	CString		strRes = GetResString(IDS_STATUS);
	HDITEM		hdi;

	hdi.mask = HDI_TEXT;
	hdi.pszText = (LPSTR)strRes.GetString();
	pHeaderCtrl->SetItem(1, &hdi);

	UpdateNickCount();

	pHeaderCtrl = serverChannelList.GetHeaderCtrl();

	for (unsigned ui = 0; ui < ELEMENT_COUNT(s_auResTbl); ui++)
	{
		::GetResString(&strRes, static_cast<UINT>(s_auResTbl[ui]));
		hdi.pszText = (LPSTR)strRes.GetString();
		pHeaderCtrl->SetItem(static_cast<int>(ui), &hdi);
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
BOOL CIrcWnd::OnInitDialog()
{
	static const uint16 s_auIconResID[] =
	{
		IDI_CHAT,
		IDI_MESSAGE,
		IDI_MPENDING
	};

	CResizableDialog::OnInitDialog();

	m_imageList.Create(16, 16, ILC_COLOR32, 0, 10);
	m_imageList.SetBkColor(RGB(255, 0, 255));
	FillImgLstWith16x16Icons(&m_imageList, s_auIconResID, ELEMENT_COUNT(s_auIconResID));

	CImageList		imageList;
	CBitmap			Bitmap, *pOldBitmap;
	CDC				*pCtrlDC = GetDC(), TempDC;

	TempDC.CreateCompatibleDC(pCtrlDC);
	Bitmap.CreateCompatibleBitmap(pCtrlDC, 16 * 3, 16);
	pOldBitmap = TempDC.SelectObject(&Bitmap);
	m_imageList.Draw(&TempDC, 0, CPoint(0, 0), ILD_NORMAL);
	m_imageList.Draw(&TempDC, 1, CPoint(16, 0), ILD_NORMAL);
	m_imageList.Draw(&TempDC, 2, CPoint(32, 0), ILD_NORMAL);
	TempDC.SelectObject(pOldBitmap);
	imageList.Create(16, 16, ILC_COLOR32 | ILC_MASK, 0, 10);
	imageList.Add(&Bitmap, RGB(255, 0, 255));
	channelselect.SetImageList(&imageList);
	imageList.Detach();
	Bitmap.DeleteObject();
	ReleaseDC(pCtrlDC);

	m_bConnected = false;
	m_bLoggedIn = false;
	Localize();
	m_pIrcMain = new CIrcMain();
	m_pIrcMain->SetIRCWnd(this);

	nickList.InsertColumn(IRC1COL_NICK, GetResString(IDS_IRC_NICK), LVCFMT_LEFT, 90);
	nickList.InsertColumn(IRC1COL_STATUS, GetResString(IDS_STATUS), LVCFMT_LEFT, 70);

	serverChannelList.InsertColumn(IRC2COL_NAME, GetResString(IDS_IRC_NAME), LVCFMT_LEFT, 203);
	serverChannelList.InsertColumn(IRC2COL_USERS, GetResString(IDS_UUSERS), LVCFMT_RIGHT, 50);
	serverChannelList.InsertColumn(IRC2COL_DESCRIPTION, GetResString(IDS_DESCRIPTION), LVCFMT_LEFT, 350);
	serverChannelList.LoadSettings(CPreferences::TABLE_IRC);

	NewChannel(GetResString(IDS_STATUS), 1);
	NewChannel(GetResString(IDS_IRC_CHANNELLIST), 2);

	UpdateFont();
	m_pCurrentChannel = (Channel*)channelPtrList.GetTail();
	AddMessage(_T(""), _T(""), 0x1000000, GetResString(IDS_IRC_STATUSLOG));
	titleWindow.SetWindowText(GetResString(IDS_IRC_CHANNELLIST));
	channelselect.SetCurSel(2);
	channelselect.SetCurFocus(2);
	OnTcnSelchangeTab2(NULL, NULL);
	channelselect.SetItemState(1, TCIS_HIGHLIGHTED, NULL);

	AddAnchor(IDC_BN_IRCCONNECT, BOTTOM_LEFT);
	AddAnchor(IDC_CLOSECHAT, BOTTOM_LEFT);
	AddAnchor(IDC_CHATSEND, BOTTOM_RIGHT);
	AddAnchor(IDC_INPUTWINDOW, BOTTOM_LEFT, BOTTOM_RIGHT);
	AddAnchor(IDC_NICKLIST, TOP_LEFT, BOTTOM_LEFT);
	AddAnchor(IDC_TITLEWINDOW, TOP_LEFT, TOP_RIGHT);
	AddAnchor(IDC_SERVERCHANNELLIST, TOP_LEFT, BOTTOM_RIGHT);
	AddAnchor(IDC_TAB2, TOP_LEFT, TOP_RIGHT);

	m_bSortAscendingChanList[IRC2COL_USERS] = false;

	nickList.SetSortArrow(IRC1COL_STATUS, m_bSortAscendingNickList[IRC1COL_STATUS]);
	nickList.SortItems(SortProcNick, IRC1COL_STATUS);

	if (g_eMuleApp.m_pGlobPrefs->DoUseSort())
		SortInit(g_eMuleApp.m_pGlobPrefs->GetIrcSortCol());
	else
	{
		int		iSortCode = g_eMuleApp.m_pGlobPrefs->GetColumnSortItem(CPreferences::TABLE_IRC);

		iSortCode |= g_eMuleApp.m_pGlobPrefs->GetColumnSortAscending(CPreferences::TABLE_IRC) ? MLC_SORTASC : MLC_SORTDESC;
		SortInit(iSortCode);
	}

	GetDlgItem(IDC_CLOSECHAT)->EnableWindow(false);
	GetDlgItem(IDC_CHATSEND)->EnableWindow(false);

	return true;
}

void CIrcWnd::OnDestroy()
{
	CResizableDialog::OnDestroy();
	m_imageList.DeleteImageList();
}

void CIrcWnd::UpdateFont()
{
	TCITEM		tcitem;
	Channel		*pChannel;
	int			i = 0;

	tcitem.mask = TCIF_PARAM;
	while (channelselect.GetItem(i++, &tcitem))
	{
		pChannel = (Channel*)tcitem.lParam;
		if (pChannel->log.m_hWnd != NULL)
			pChannel->log.SetFont(&g_eMuleApp.m_pdlgEmule->m_fontDefault);
	}
	titleWindow.SetFont(&g_eMuleApp.m_pdlgEmule->m_fontDefault);
	inputWindow.SetFont(&g_eMuleApp.m_pdlgEmule->m_fontDefault);
}

void CIrcWnd::OnSize(UINT nType, int cx, int cy)
{
	CResizableDialog::OnSize(nType, cx, cy);

	if (m_pCurrentChannel != NULL && m_pCurrentChannel->log.m_hWnd)
	{
		CRect		rcChannel;

		serverChannelList.GetWindowRect(&rcChannel);
		ScreenToClient(&rcChannel);
		m_pCurrentChannel->log.SetWindowPos(NULL, rcChannel.left, rcChannel.top, rcChannel.Width(), rcChannel.Height(), SWP_NOZORDER);
	}
}

int CIrcWnd::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	return CResizableDialog::OnCreate(lpCreateStruct);
}

void CIrcWnd::DoDataExchange(CDataExchange* pDX)
{
	CResizableDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_NICKLIST, nickList);
	DDX_Control(pDX, IDC_INPUTWINDOW, inputWindow);
	DDX_Control(pDX, IDC_TITLEWINDOW, titleWindow);
	DDX_Control(pDX, IDC_SERVERCHANNELLIST, serverChannelList);
	DDX_Control(pDX, IDC_TAB2, channelselect);
}

BOOL CIrcWnd::OnCommand(WPARAM wParam, LPARAM lParam)
{
	int			nickItem = nickList.GetNextItem(-1, LVIS_SELECTED | LVIS_FOCUSED);
	int			chanItem = channelselect.GetCurSel();
//	int			chanLItem= serverChannelList.GetNextItem(-1, LVIS_SELECTED | LVIS_FOCUSED);
	Nick		*pNick;
	TCITEM		tcitem;
	Channel		*pChannel;
	CString		strSend;

	switch (wParam)
	{
		case IDC_BN_IRCCONNECT:
		{
			OnBnClickedBnIrcconnect();
			return true;
		}
		case IDC_CHATSEND:
		{
			OnBnClickedChatsend();
			return true;
		}
		case IDC_CLOSECHAT:
		{
			OnBnClickedClosechat();
			return true;
		}
		case Irc_Priv:
		{
			pNick = (Nick*)nickList.GetItemData(nickItem);
			if (pNick != NULL)
				AddMessage(pNick->nick, _T(""), RGB(0, 147, 0), GetResString(IDS_IRC_PRIVATECHANSTART));
			return true;
		}
		case Irc_Owner:
		{
			pNick = (Nick*)nickList.GetItemData(nickItem);
			tcitem.mask = TCIF_PARAM;
			channelselect.GetItem(chanItem, &tcitem);
			pChannel = (Channel*)tcitem.lParam;
			if (pNick != NULL && pChannel != NULL)
			{
				strSend.Format(_T("PRIVMSG chanserv owner %s %s"), pChannel->name, pNick->nick);
				m_pIrcMain->SendString(strSend);
			}
			return true;
		}
		case Irc_DeOwner:
		{
			pNick = (Nick*)nickList.GetItemData(nickItem);
			tcitem.mask = TCIF_PARAM;
			channelselect.GetItem(chanItem, &tcitem);
			pChannel = (Channel*)tcitem.lParam;
			if (pNick != NULL && pChannel != NULL)
			{
				strSend.Format(_T("PRIVMSG chanserv deowner %s %s"), pChannel->name, pNick->nick);
				m_pIrcMain->SendString(strSend);
			}
			return true;
		}
		case Irc_Op:
		{
			pNick = (Nick*)nickList.GetItemData(nickItem);
			tcitem.mask = TCIF_PARAM;
			channelselect.GetItem(chanItem, &tcitem);
			pChannel = (Channel*)tcitem.lParam;
			if (pNick != NULL && pChannel != NULL)
			{
				strSend.Format(_T("MODE %s +o %s"), pChannel->name, pNick->nick);
				m_pIrcMain->SendString(strSend);
			}
			return true;
		}
		case Irc_DeOp:
		{
			pNick = (Nick*)nickList.GetItemData(nickItem);
			tcitem.mask = TCIF_PARAM;
			channelselect.GetItem(chanItem, &tcitem);
			pChannel = (Channel*)tcitem.lParam;
			if (pNick != NULL && pChannel != NULL)
			{
				strSend.Format(_T("MODE %s -o %s"), pChannel->name, pNick->nick);
				m_pIrcMain->SendString(strSend);
			}
			return true;
		}
		case Irc_HalfOp:
		{
			pNick = (Nick*)nickList.GetItemData(nickItem);
			tcitem.mask = TCIF_PARAM;
			channelselect.GetItem(chanItem, &tcitem);
			pChannel = (Channel*)tcitem.lParam;
			if (pNick != NULL && pChannel != NULL)
			{
				strSend.Format(_T("MODE %s +h %s"), pChannel->name, pNick->nick);
				m_pIrcMain->SendString(strSend);
			}
			return true;
		}
		case Irc_DeHalfOp:
		{
			pNick = (Nick*)nickList.GetItemData(nickItem);
			tcitem.mask = TCIF_PARAM;
			channelselect.GetItem(chanItem, &tcitem);
			pChannel = (Channel*)tcitem.lParam;
			if (pNick != NULL && pChannel != NULL)
			{
				strSend.Format(_T("MODE %s -h %s"), pChannel->name, pNick->nick);
				m_pIrcMain->SendString(strSend);
			}
			return true;
		}
		case Irc_Voice:
		{
			pNick = (Nick*)nickList.GetItemData(nickItem);
			tcitem.mask = TCIF_PARAM;
			channelselect.GetItem(chanItem, &tcitem);
			pChannel = (Channel*)tcitem.lParam;
			if (pNick != NULL && pChannel != NULL)
			{
				strSend.Format(_T("MODE %s +v %s"), pChannel->name, pNick->nick);
				m_pIrcMain->SendString(strSend);
			}
			return true;
		}
		case Irc_DeVoice:
		{
			pNick = (Nick*)nickList.GetItemData(nickItem);
			tcitem.mask = TCIF_PARAM;
			channelselect.GetItem(chanItem, &tcitem);
			pChannel = (Channel*)tcitem.lParam;
			if (pNick != NULL && pChannel != NULL)
			{
				strSend.Format(_T("MODE %s -v %s"), pChannel->name, pNick->nick);
				m_pIrcMain->SendString(strSend);
			}
			return true;
		}
		case Irc_Protect:
		{
			pNick = (Nick*)nickList.GetItemData(nickItem);
			tcitem.mask = TCIF_PARAM;
			channelselect.GetItem(chanItem, &tcitem);
			pChannel = (Channel*)tcitem.lParam;
			if (pNick != NULL && pChannel != NULL)
			{
				strSend.Format(_T("PRIVMSG chanserv protect %s %s"), pChannel->name, pNick->nick);
				m_pIrcMain->SendString(strSend);
			}
			return true;
		}
		case Irc_DeProtect:
		{
			pNick = (Nick*)nickList.GetItemData(nickItem);
			tcitem.mask = TCIF_PARAM;
			channelselect.GetItem(chanItem, &tcitem);
			pChannel = (Channel*)tcitem.lParam;
			if (pNick != NULL && pChannel != NULL)
			{
				strSend.Format(_T("PRIVMSG chanserv deprotect %s %s"), pChannel->name, pNick->nick);
				m_pIrcMain->SendString(strSend);
			}
			return true;
		}
		case Irc_Kick:
		{
			pNick = (Nick*)nickList.GetItemData(nickItem);
			tcitem.mask = TCIF_PARAM;
			channelselect.GetItem(chanItem, &tcitem);
			pChannel = (Channel*)tcitem.lParam;
			if (pNick != NULL && pChannel != NULL)
			{
				strSend.Format(_T("KICK %s %s"), pChannel->name, pNick->nick);
				m_pIrcMain->SendString(strSend);
			}
			return true;
		}
		case Irc_Slap:
		{
			pNick = (Nick*)nickList.GetItemData(nickItem);
			tcitem.mask = TCIF_PARAM;
			channelselect.GetItem(chanItem, &tcitem);
			pChannel = (Channel*)tcitem.lParam;
			if (pNick != NULL && pChannel != NULL)
			{
				strSend.Format(GetResString(IDS_IRC_SLAPMSGSEND), pChannel->name, pNick->nick);
				AddMessage(pChannel->name, _T(""), RGB(252, 127, 0), GetResString(IDS_IRC_SLAPMSG), m_pIrcMain->GetNick(), pNick->nick);
				m_pIrcMain->SendString(strSend);
			}
			return true;
		}
		case Irc_AddFriend:
		{
			pNick = (Nick*)nickList.GetItemData(nickItem);
			tcitem.mask = TCIF_PARAM;
			channelselect.GetItem(chanItem, &tcitem);
			pChannel = (Channel*)tcitem.lParam;
			if (pNick != NULL && pChannel != NULL)
			{
				strSend.Format(_T("PRIVMSG %s :\001RQSFRIEND |%i|\001"), pNick->nick, m_pIrcMain->SetVerify());
				m_pIrcMain->SendString(strSend);
			}
			return true;
		}
		case Irc_SendLink:
		{
		   	if (!GetSendFileString())
				   return true;

			pNick = (Nick*)nickList.GetItemData(nickItem);
			tcitem.mask = TCIF_PARAM;
			channelselect.GetItem(chanItem, &tcitem);
			pChannel = (Channel*)tcitem.lParam;
			if (pNick != NULL && pChannel != NULL)
			{
				strSend.Format(_T("PRIVMSG %s :\001SENDLINK|%s|%s\001"), pNick->nick, HashToString(g_eMuleApp.m_pGlobPrefs->GetUserHash()), GetSendFileString() );
				m_pIrcMain->SendString(strSend);
			}
			return true;
		}

		case Irc_Close:
		{
			OnBnClickedClosechat();
			return true;
		}
		case Irc_Join:
		{
			JoinChannels();
			return true;
		}
	}
	return true;
}

BOOL CIrcWnd::PreTranslateMessage(MSG* pMsg)
{
	if (pMsg->message == WM_KEYDOWN && (pMsg->hwnd == GetDlgItem(IDC_INPUTWINDOW)->m_hWnd))
	{
		if (pMsg->wParam == VK_RETURN)
		{
			OnBnClickedChatsend();
			return TRUE;
		}

		if (pMsg->wParam == VK_UP || pMsg->wParam == VK_DOWN)
		{
			ScrollHistory(pMsg->wParam == VK_DOWN);
			return TRUE;
		}
	}

	return CResizableDialog::PreTranslateMessage(pMsg);
}

void CIrcWnd::OnBnClickedBnIrcconnect()
{
	if (!m_bConnected)
		m_pIrcMain->Connect();
	else
		m_pIrcMain->Disconnect();
	GetDlgItem(IDC_INPUTWINDOW)->SetFocus();
}

void CIrcWnd::OnBnClickedClosechat(int nItem)
{
	TCITEM		tcitem;

	tcitem.mask = TCIF_PARAM;
	if (nItem == -1)
		nItem = channelselect.GetCurSel();

	if (nItem == -1)
		return;

	channelselect.GetItem(nItem, &tcitem);

	Channel		*pPartChannel = (Channel*)tcitem.lParam;

	if (pPartChannel != NULL)
	{
		if (pPartChannel->type == 4 && m_bConnected)
		{
			CString		strPart = _T("PART ");

			strPart += pPartChannel->name;
			m_pIrcMain->SendString(strPart);
			return;
		}
		else if (pPartChannel->type == 5 || pPartChannel->type == 4)
		{
			RemoveChannel(pPartChannel->name);
			return;
		}
	}
}

void CIrcWnd::OnEnLinkIrcBox(NMHDR *pNMHDR, LRESULT *pResult)
{
	*pResult = 0;

	ENLINK		*pEnLink = reinterpret_cast<ENLINK *>(pNMHDR);
	TCITEM		tcitem;

	tcitem.mask = TCIF_PARAM;

	int			iCurSel = channelselect.GetCurSel();

	if (iCurSel != -1 && pEnLink && pEnLink->msg == WM_LBUTTONDOWN && channelselect.GetItem(iCurSel, &tcitem))
	{
		Channel		*pUpdate = (Channel*)tcitem.lParam;
		CString		strUrl;

		pUpdate->log.GetTextRange(pEnLink->chrg.cpMin, pEnLink->chrg.cpMax, strUrl);
		ShellExecute(NULL, NULL, strUrl, NULL, NULL, SW_SHOWDEFAULT);
		*pResult = 1;
	}
}

void CIrcWnd::OnTcnSelchangeTab2(NMHDR *pNMHDR, LRESULT *pResult)
{
	nickList.DeleteAllItems();

	TCITEM		tcitem;

	tcitem.mask = TCIF_PARAM;

	int		iCurSel = channelselect.GetCurSel();

	if (iCurSel == -1)
		return;

	if (!channelselect.GetItem(iCurSel, &tcitem))
		return;

	Channel		*pUpdate = (Channel*)tcitem.lParam, *pChannel;
	int			i;
	bool		bWasChanged = (channelselect.GetItemState(iCurSel, TCIS_HIGHLIGHTED) & TCIS_HIGHLIGHTED);

	m_pCurrentChannel = pUpdate;
	UpdateNickCount();
	channelselect.SetItemState(iCurSel, TCIS_HIGHLIGHTED, NULL);
	if (m_pCurrentChannel->type == 1)
	{
		titleWindow.SetWindowText(GetResString(IDS_STATUS));
		GetDlgItem(IDC_CLOSECHAT)->EnableWindow(false);
	}
	else if (m_pCurrentChannel->type == 2)
	{
		titleWindow.SetWindowText(GetResString(IDS_IRC_CHANNELLIST));
		serverChannelList.ShowWindow(SW_SHOW);
		GetDlgItem(IDC_CLOSECHAT)->EnableWindow(false);
		tcitem.mask = TCIF_PARAM;
		i = 0;
		while (channelselect.GetItem(i++, &tcitem))
		{
			pChannel = (Channel*)tcitem.lParam;
			if (pChannel != m_pCurrentChannel && pChannel->log.m_hWnd != NULL)
				pChannel->log.ShowWindow(SW_HIDE);
		}
		return;
	}
	else
		GetDlgItem(IDC_CLOSECHAT)->EnableWindow(true);
	SetActivity(m_pCurrentChannel->name, false);

	CRect		rcChannel;

	serverChannelList.GetWindowRect(&rcChannel);
	ScreenToClient(&rcChannel);
	m_pCurrentChannel->log.SetWindowPos(NULL, rcChannel.left, rcChannel.top, rcChannel.Width(), rcChannel.Height(), SWP_NOZORDER);
	m_pCurrentChannel->log.SetRedraw(false);
	m_pCurrentChannel->log.ShowWindow(SW_SHOW);
	if (m_pCurrentChannel->log.m_bAutoScroll && bWasChanged)
		m_pCurrentChannel->log.ScrollToLastLine();
	m_pCurrentChannel->log.SetRedraw(true);

	tcitem.mask = TCIF_PARAM;
	i = 0;
	while (channelselect.GetItem(i++, &tcitem))
	{
		pChannel = (Channel*)tcitem.lParam;
		if (pChannel != m_pCurrentChannel && pChannel->log.m_hWnd != NULL)
			pChannel->log.ShowWindow(SW_HIDE);
	}
	serverChannelList.ShowWindow(SW_HIDE);
	RefreshNickList(pUpdate->name);
	SetTitle(pUpdate->name, pUpdate->title);
	pUpdate->log.SetTitle(pUpdate->name);
	GetDlgItem(IDC_INPUTWINDOW)->SetFocus();
	if (pResult != NULL)
		*pResult = 0;
}

BEGIN_MESSAGE_MAP(CIrcWnd, CDialog)
	ON_NOTIFY(EN_LINK, 125, OnEnLinkIrcBox)
	ON_NOTIFY(NM_DBLCLK, IDC_SERVERCHANNELLIST, OnNMDblclkserverChannelList)
	ON_NOTIFY(NM_DBLCLK, IDC_NICKLIST, OnNMDblclkNickList)
	ON_NOTIFY(LVN_COLUMNCLICK, IDC_NICKLIST, OnColumnClickNick)
	ON_NOTIFY(LVN_COLUMNCLICK, IDC_SERVERCHANNELLIST, OnColumnClickChanL)
	ON_NOTIFY(NM_RCLICK, IDC_NICKLIST, OnNMRclickNick)
	ON_NOTIFY(NM_RCLICK, IDC_SERVERCHANNELLIST, OnNMRclickChanL)
	ON_NOTIFY(TCN_SELCHANGE, IDC_TAB2, OnTcnSelchangeTab2)
	ON_MESSAGE(WM_CLOSETAB, OnCloseTab)
	ON_WM_SIZE()
	ON_WM_CREATE()
	ON_WM_DESTROY()
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////
//	Channel List
/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////

void CIrcWnd::ResetServerChannelList()
{
	POSITION		pos1, pos2;
	ChannelList		*pCurChannel;

	for (pos1 = channelLPtrList.GetHeadPosition(); (pos2 = pos1) != NULL;)
	{
		pCurChannel = (ChannelList*)channelLPtrList.GetNext(pos1);
		channelLPtrList.RemoveAt(pos2);
		delete pCurChannel;
	}
	serverChannelList.DeleteAllItems();
}

void CIrcWnd::AddChannelToList(CString name, CString user, CString description)
{
	CString		ntemp = name;
	CString		dtemp = description;
	int			iUserTest = _tstoi(user);

	if ((g_eMuleApp.m_pGlobPrefs->GetIRCChanNameFilter() || g_eMuleApp.m_pGlobPrefs->GetIRCChannelUserFilter()) && g_eMuleApp.m_pGlobPrefs->GetIRCUseChanFilter())
	{
		if (iUserTest < g_eMuleApp.m_pGlobPrefs->GetIRCChannelUserFilter())
			return;
		if (dtemp.MakeLower().Find(g_eMuleApp.m_pGlobPrefs->GetIRCChanNameFilter().MakeLower()) == -1 && ntemp.MakeLower().Find(g_eMuleApp.m_pGlobPrefs->GetIRCChanNameFilter().MakeLower()) == -1)
			return;
	}
	ChannelList		*pToAdd = new ChannelList;

	pToAdd->name = name;
	pToAdd->users = user;
	pToAdd->desc = StripMessageOfFontCodes(description);
	channelLPtrList.AddTail(pToAdd);

	int			iItems = serverChannelList.InsertItem(LVIF_PARAM, serverChannelList.GetItemCount(), 0, 0, 0, 0, (LPARAM)pToAdd);

	serverChannelList.SetItemText(iItems, 0, pToAdd->name);
	serverChannelList.SetItemText(iItems, 1, pToAdd->users);
	serverChannelList.SetItemText(iItems, 2, pToAdd->desc);
	if (channelselect.GetCurSel() != 1)
		channelselect.SetItemState(1, TCIS_HIGHLIGHTED, TCIS_HIGHLIGHTED);
}

int CIrcWnd::SortProcChanL(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort)
{
	ChannelList		*pItem1 = (ChannelList*)lParam1;
	ChannelList		*pItem2 = (ChannelList*)lParam2;

	switch (lParamSort)
	{
		case IRC2COL_NAME:
			return CString(pItem1->name).CompareNoCase(pItem2->name);
		case IRC2COL_NAME + MLC_SORTDESC:
			return CString(pItem2->name).CompareNoCase(pItem1->name);
		case IRC2COL_USERS:
			return _tstoi(pItem1->users) - _tstoi(pItem2->users);
		case IRC2COL_USERS + MLC_SORTDESC:
			return _tstoi(pItem2->users) - _tstoi(pItem1->users);
		case IRC2COL_DESCRIPTION:
			return CString(pItem1->desc).CompareNoCase(pItem2->desc);
		case IRC2COL_DESCRIPTION + MLC_SORTDESC:
			return CString(pItem2->desc).CompareNoCase(pItem1->desc);
		default:
			return 0;
	}
}

void CIrcWnd::OnColumnClickChanL(NMHDR* pNMHDR, LRESULT* pResult)
{
	NM_LISTVIEW		*pNMListView = reinterpret_cast<NM_LISTVIEW*>(pNMHDR);
	int				iSubItem = pNMListView->iSubItem;
	bool			bSortOrder = m_bSortAscendingChanList[iSubItem];

// Reverse sorting direction for the same column and keep the same if column was changed
	if ((serverChannelList.GetSortParam() & MLC_COLUMNMASK) == iSubItem)
		m_bSortAscendingChanList[iSubItem] = bSortOrder = !bSortOrder;

	serverChannelList.SetSortArrow(iSubItem, bSortOrder);
	serverChannelList.SortItems(SortProcChanL, iSubItem + ((bSortOrder) ? MLC_SORTASC : MLC_SORTDESC));
	g_eMuleApp.m_pGlobPrefs->SetColumnSortItem(CPreferences::TABLE_IRC, iSubItem);
	g_eMuleApp.m_pGlobPrefs->SetColumnSortAscending(CPreferences::TABLE_IRC, bSortOrder);
	*pResult = 0;
}

void CIrcWnd::SortInit(int iSortCode)
{
//	Get the sort column
	int		iSortColumn = (iSortCode & MLC_COLUMNMASK);
//	Get the sort order
	bool	bSortAscending = (iSortCode & MLC_SORTDESC) == 0;

	serverChannelList.SetSortArrow(iSortColumn, bSortAscending);
	serverChannelList.SortItems(SortProcChanL, iSortCode);
	m_bSortAscendingChanList[iSortColumn] = bSortAscending;
}

void CIrcWnd::OnNMRclickChanL(NMHDR *pNMHDR, LRESULT *pResult)
{
	CTitleMenu	menuChanL;
	POINT		point;

	::GetCursorPos(&point);
	
	menuChanL.CreatePopupMenu();
	menuChanL.AddMenuTitle(GetResString(IDS_IRC_CHANNEL));
	menuChanL.AppendMenu(MF_STRING, Irc_Join, GetResString(IDS_IRC_JOIN));
	menuChanL.TrackPopupMenuEx(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y, this, NULL);

//	Menu objects are destroyed in their destructor
	*pResult = 0;
}

void CIrcWnd::OnNMDblclkserverChannelList(NMHDR *pNMHDR, LRESULT *pResult)
{
	JoinChannels();
	*pResult = 0;
}

/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////
//	Nick List
/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////

Nick* CIrcWnd::FindNickByName(CString channel, CString name)
{
	Channel		*pCurChannel = FindChannelByName(channel);
	Nick		*pCurNick;

	if (pCurChannel == NULL)
		return 0;

	for (POSITION pos = pCurChannel->nicks.GetHeadPosition(); pos != NULL;)
	{
		pCurNick = (Nick*)pCurChannel->nicks.GetNext(pos);
		if (pCurNick->nick == name)
			return pCurNick;
	}
	return 0;
}

Nick* CIrcWnd::NewNick(CString channel, CString nick)
{
	Channel		*pToAddChan = FindChannelByName(channel);

	if (pToAddChan == NULL)
		return NULL;

	if (FindNickByName(channel, nick) != NULL)
		return NULL;

	Nick		*pToAddNick = new Nick;
	
	if (nick.Left(1) == _T("!"))
	{
		nick = nick.Mid(1);
		pToAddNick->owner = _T("!");
	}
	else
		pToAddNick->owner.Empty();

	if (nick.Left(1) == _T("*"))
	{
		nick = nick.Mid(1);
		pToAddNick->protect = _T("*");
	}
	else
		pToAddNick->protect.Empty();

	if (nick.Left(1) == _T("@"))
	{
		pToAddNick->op = _T("@");
		pToAddNick->hop.Empty();
		pToAddNick->voice.Empty();
		pToAddNick->uop.Empty();
		pToAddNick->nick = nick.Mid(1);
	}
	else if (nick.Left(1) == _T("%"))
	{
		pToAddNick->op.Empty();
		pToAddNick->hop = _T("%");
		pToAddNick->voice.Empty();
		pToAddNick->uop.Empty();
		pToAddNick->nick = nick.Mid(1);
	}
	else if (nick.Left(1) == _T("+"))
	{
		pToAddNick->op.Empty();
		pToAddNick->hop.Empty();
		pToAddNick->voice = _T("+");
		pToAddNick->uop.Empty();
		pToAddNick->nick = nick.Mid(1);
	}
	else if (nick.Left(1) == _T("-"))
	{
		pToAddNick->op.Empty();
		pToAddNick->hop.Empty();
		pToAddNick->voice.Empty();
		pToAddNick->uop = _T("-");
		pToAddNick->nick = nick.Mid(1);
	}
	else
	{
		pToAddNick->op.Empty();
		pToAddNick->hop.Empty();
		pToAddNick->voice.Empty();
		pToAddNick->nick = nick;
	}
	pToAddNick->nick.TrimLeft(_T("*@%+-"));
	pToAddChan->nicks.AddTail(pToAddNick);
	if (pToAddChan == m_pCurrentChannel)
	{
		CString		strMode;
		int			iItems = nickList.InsertItem(LVIF_PARAM, nickList.GetItemCount(), 0, 0, 0, 0, (LPARAM)pToAddNick);

		nickList.SetItemText(iItems, 0, (LPCTSTR)pToAddNick->nick);
		strMode.Format(_T("%s%s%s%s%s%s"), pToAddNick->owner, pToAddNick->protect, pToAddNick->op, pToAddNick->hop, pToAddNick->voice, pToAddNick->uop);
		nickList.SetItemText(iItems, 1, (LPCTSTR)strMode);
		UpdateNickCount();
	}
	return pToAddNick;
}

void CIrcWnd::RefreshNickList(CString channel)
{
	nickList.DeleteAllItems();

	Channel		*pRefresh = FindChannelByName(channel);

	if (pRefresh == NULL)
		return;

	Nick		*pCurNick;
	int			iItems;
	CString		strMode;

	for (POSITION pos = pRefresh->nicks.GetHeadPosition(); pos != NULL;)
	{
        pCurNick = (Nick*)pRefresh->nicks.GetNext(pos);
		iItems = nickList.InsertItem(LVIF_PARAM, nickList.GetItemCount(), 0, 0, 0, 0, (LPARAM)pCurNick);
		nickList.SetItemText(iItems, 0, (LPCTSTR)pCurNick->nick);
		strMode.Format(_T("%s%s%s%s%s%s"), pCurNick->owner, pCurNick->protect, pCurNick->op, pCurNick->hop, pCurNick->voice, pCurNick->uop);
		nickList.SetItemText(iItems, 1, (LPCTSTR)strMode);
	}
	UpdateNickCount();
}

bool CIrcWnd::RemoveNick(CString channel, CString nick)
{
	Channel		*pUpdate = FindChannelByName(channel);

	if (pUpdate == NULL)
		return false;

	POSITION		pos1, pos2;
	Nick			*pCurNick;
	LVFINDINFO		lvdfind;
	sint32			iResult;

	for (pos1 = pUpdate->nicks.GetHeadPosition(); (pos2 = pos1) != NULL;)
	{
		pCurNick = (Nick*)pUpdate->nicks.GetNext(pos1);
		if (pCurNick->nick == nick)
		{
			if (pUpdate == m_pCurrentChannel)
			{
				lvdfind.flags = LVFI_PARAM;
				lvdfind.lParam = (LPARAM)pCurNick;
				iResult = nickList.FindItem(&lvdfind);
				if (iResult != -1)
				{
					nickList.DeleteItem(iResult);
					UpdateNickCount();
				}
			}
			pUpdate->nicks.RemoveAt(pos2);
			delete pCurNick;
			return true;
		}
	}
	return false;
}

void CIrcWnd::DeleteAllNick(CString channel)
{
	Channel		*pCurChannel = FindChannelByName(channel);

	if (pCurChannel == NULL)
		return;

	POSITION	pos1, pos2;
	Nick		*pCurNick;

	for (pos1 = pCurChannel->nicks.GetHeadPosition(); (pos2 = pos1) != NULL;)
	{
		pCurNick = (Nick*)pCurChannel->nicks.GetNext(pos1);
		pCurChannel->nicks.RemoveAt(pos2);
		delete pCurNick;
	}
}

void CIrcWnd::DeleteNickInAll(CString nick, CString message)
{
	Channel		*pCurChannel;
	CString		strMessage, strLine;

	for (POSITION pos = channelPtrList.GetHeadPosition(); pos != NULL;)
	{
		pCurChannel = (Channel*)channelPtrList.GetNext(pos);
		if (RemoveNick(pCurChannel->name, nick))
		{
			if (!g_eMuleApp.m_pGlobPrefs->GetIrcIgnoreInfoMessage())
			{
				strMessage = message;
				strMessage.Trim();
				strLine.Format(GetResString(IDS_IRC_HASQUIT), nick, strMessage);
				if (strMessage.GetLength() == 0)
					strLine.Truncate(strLine.GetLength() - 3);
				AddMessage(pCurChannel->name, _T(""), RGB(51, 102, 255), strLine);
			}
		}
	}
}

bool CIrcWnd::ChangeNick(CString channel, CString oldnick, CString newnick)
{
	Channel		*pUpdate = FindChannelByName(channel);

	if (pUpdate == NULL)
		return false;

	Nick			*pCurNick;
	LVFINDINFO		lvfind;
	sint32			iResult;

	for (POSITION pos = pUpdate->nicks.GetHeadPosition(); pos != NULL;)
	{
		pCurNick = (Nick*)pUpdate->nicks.GetNext(pos);
		if (pCurNick->nick == oldnick)
		{
			if ((pUpdate = m_pCurrentChannel) != NULL)
			{
				lvfind.flags = LVFI_PARAM;
				lvfind.lParam = (LPARAM)pCurNick;
				iResult = nickList.FindItem(&lvfind);
				if (iResult != -1)
					nickList.SetItemText(iResult, 0, (LPCTSTR)newnick);
			}
			pCurNick->nick = newnick;
			return true;
		}
	}
	return false;
}

bool CIrcWnd::ChangeMode(CString channel, CString nick, CString mode)
{
	Channel		*pUpdate = FindChannelByName(channel);

	if (pUpdate == NULL)
		return false;

	Nick			*pCurNick;
	LVFINDINFO		lvfind;
	sint32			iResult;
	CString			strMode;

	for (POSITION pos = pUpdate->nicks.GetHeadPosition(); pos != NULL;)
	{
		pCurNick = (Nick*)pUpdate->nicks.GetNext(pos);
		if (pCurNick->nick == nick)
		{
			if (mode == _T("+a"))
				pCurNick->protect = _T("*");
			else if (mode == _T("-a"))
				pCurNick->protect.Empty();
			else if (mode == _T("+h"))
				pCurNick->hop = _T("%");
			else if (mode == _T("-h"))
				pCurNick->hop.Empty();
			else if (mode == _T("+o"))
				pCurNick->op = _T("@");
			else if (mode == _T("-o"))
				pCurNick->op.Empty();
			else if (mode == _T("+q"))
				pCurNick->owner = _T("!");
			else if (mode == _T("-q"))
				pCurNick->owner.Empty();
			else if (mode == _T("+u"))
				pCurNick->uop = _T("-");
			else if (mode == _T("-u"))
				pCurNick->uop.Empty();
			else if (mode == _T("+v"))
				pCurNick->voice = _T("+");
			else if (mode == _T("-v"))
				pCurNick->voice.Empty();
			if (pUpdate = m_pCurrentChannel)
			{
				lvfind.flags = LVFI_PARAM;
				lvfind.lParam = (LPARAM)pCurNick;
				iResult = nickList.FindItem(&lvfind);
				if (iResult != -1)
				{
					strMode.Format(_T("%s%s%s%s%s%s"), pCurNick->owner, pCurNick->protect, pCurNick->op, pCurNick->hop, pCurNick->voice, pCurNick->uop);
					nickList.SetItemText(iResult, 1, (LPCTSTR)strMode);
				}
			}
		}
	}
	return true;
}

void CIrcWnd::ParseChangeMode(CString channel, CString changer, CString commands, CString names)
{
	try
	{
		if (commands.GetLength() == 2)
		{
			if (ChangeMode(channel, names, commands))
				if (!g_eMuleApp.m_pGlobPrefs->GetIrcIgnoreInfoMessage())
					AddMessage(channel, _T(""), RGB(0, 147, 0), GetResString(IDS_IRC_SETSMODE), changer, commands, names);
			return;
		}
		else
		{
			CString		strDir;

			strDir = commands[0];
			if (strDir == _T("+") || strDir == _T("-"))
			{
				int		iCurMode = 1;
				int		iCurName = 0;
				int		iCurNameBack = names.Find(_T(' '), iCurName);

				while (iCurMode < commands.GetLength())
				{
					CString		strTest;

					if(iCurNameBack > iCurName)
					{
						strTest = names.Mid(iCurName, iCurNameBack - iCurName);
						iCurName = iCurNameBack + 1;
					}
					if (ChangeMode(channel, strTest, strDir + commands[iCurMode]) && !g_eMuleApp.m_pGlobPrefs->GetIrcIgnoreInfoMessage())
						AddMessage(channel, _T(""), RGB(0, 147, 0), GetResString(IDS_IRC_SETSMODE), changer, strDir + commands[iCurMode] , strTest);
					iCurNameBack = names.Find(_T(' '), iCurName + 1);
					if (iCurNameBack == -1)
						iCurNameBack = names.GetLength();
					iCurMode++;
				}
			}
		}
	}
	catch (...)
	{
		AddMessage(channel, _T(""), RGB(252, 127, 0), GetResString(IDS_IRC_NOTSUPPORTED));
	}
}

void CIrcWnd::ChangeAllNick(CString oldnick, CString newnick)
{
	Channel		*pCurChannel = FindChannelByName(oldnick);

	if (pCurChannel != NULL)
	{
		pCurChannel->name = newnick;

		TCITEM		tcitem;
		int			i;

		tcitem.mask = TCIF_PARAM;
		tcitem.lParam = -1;
		for (i = 0; i < channelselect.GetItemCount();i++)
		{
			channelselect.GetItem(i, &tcitem);
			if (((Channel*)tcitem.lParam) == pCurChannel)
				break;
		}
		if (((Channel*)tcitem.lParam) != pCurChannel)
			return;
		tcitem.mask = TCIF_TEXT;
		tcitem.pszText = newnick.GetBuffer();
		tcitem.cchTextMax = (int)newnick.GetLength() + 1;
		channelselect.SetItem(i, &tcitem);
	}
	for (POSITION pos = channelPtrList.GetHeadPosition(); pos != NULL;)
	{
		pCurChannel = (Channel*)channelPtrList.GetNext(pos);
		if (ChangeNick(pCurChannel->name, oldnick, newnick) && !g_eMuleApp.m_pGlobPrefs->GetIrcIgnoreInfoMessage())
			AddMessage(pCurChannel->name, _T(""), RGB(0, 147, 0), GetResString(IDS_IRC_NOWKNOWNAS), oldnick, newnick);
	}
}

/*
void CIrcWnd::SetNick( CString in_nick )
{
	g_eMuleApp.m_pGlobPrefs->SetIRCNick( in_nick.GetBuffer() );
	//Need to also update the preference window for this to work right..
}
*/

int CIrcWnd::SortProcNick(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort)
{
	Nick	*pItem1 = (Nick*)lParam1;
	Nick	*pItem2 = (Nick*)lParam2;

	switch (lParamSort)
	{
	case IRC1COL_NICK:
		return pItem1->nick.CompareNoCase(pItem2->nick);
	case IRC1COL_NICK + MLC_SORTDESC:
		return pItem2->nick.CompareNoCase(pItem1->nick);
	case IRC1COL_STATUS:
		if (pItem1->owner == _T("!"))
		{
			if (pItem2->owner != _T("!"))
				return -1;
		}
		else if (pItem2->owner == _T("!"))
		{
			return 1;
		}
		if (pItem1->protect == _T("*"))
		{
			if (pItem2->protect != _T("*"))
				return -1;
		}
		else if (pItem2->protect == _T("*"))
		{
			return 1;
		}
		if (pItem1->op == _T("@"))
		{
			if (pItem2->op != _T("@"))
				return -1;
		}
		else if (pItem2->op == _T("@"))
		{
			return 1;
		}
		if (pItem1->hop == _T("%"))
		{
			if (pItem2->hop != _T("%"))
				return -1;
		}
		else if (pItem2->hop == _T("%"))
		{
			return 1;
		}
		if (pItem1->voice == _T("+"))
		{
			if (pItem2->voice != _T("+"))
				return -1;
		}
		else if (pItem2->voice == _T("+"))
		{
			return 1;
		}
		if (pItem1->uop == _T("-"))
		{
			if (pItem2->uop != _T("-"))
				return -1;
		}
		else if (pItem2->uop == _T("-"))
		{
			return 1;
		}
		return pItem1->nick.CompareNoCase(pItem2->nick);

	case IRC1COL_STATUS + MLC_SORTDESC:
		if (pItem2->owner == _T("!"))
		{
			if (pItem1->owner != _T("!"))
				return -1;
		}
		else if (pItem1->owner == _T("!"))
		{
			return 1;
		}
		if (pItem2->protect == _T("*"))
		{
			if (pItem1->protect != _T("*"))
				return -1;
		}
		else if (pItem1->protect == _T("*"))
		{
			return 1;
		}
		if (pItem2->op == _T("@"))
		{
			if (pItem1->op != _T("@"))
				return -1;
		}
		else if (pItem1->op == _T("@"))
		{
			return 1;
		}
		if (pItem2->hop == _T("%"))
		{
			if (pItem1->hop != _T("%"))
				return -1;
		}
		else if (pItem1->hop == _T("%"))
		{
			return 1;
		}
		if (pItem2->voice == _T("+"))
		{
			if (pItem1->voice != _T("+"))
				return -1;
		}
		else if (pItem1->voice == _T("+"))
		{
			return 1;
		}
		if (pItem2->uop == _T("-"))
		{
			if (pItem1->uop != _T("-"))
				return -1;
		}
		else if (pItem1->uop == _T("-"))
		{
			return 1;
		}
		return pItem2->nick.CompareNoCase(pItem1->nick);
	default:
		return 0;
	}
}

void CIrcWnd::OnColumnClickNick(NMHDR* pNMHDR, LRESULT* pResult)
{
	NM_LISTVIEW		*pNMListView = reinterpret_cast<NM_LISTVIEW*>(pNMHDR);
	int				iSubItem = pNMListView->iSubItem;
	bool			bSortOrder = m_bSortAscendingNickList[iSubItem];

// Reverse sorting direction for the same column and keep the same if column was changed
	if ((nickList.GetSortParam() & MLC_COLUMNMASK) == iSubItem)
		m_bSortAscendingNickList[iSubItem] = bSortOrder = !bSortOrder;

	nickList.SetSortArrow(iSubItem, bSortOrder);
	nickList.SortItems(SortProcNick, iSubItem + ((bSortOrder) ? MLC_SORTASC : MLC_SORTDESC));
	*pResult = 0;
}

void CIrcWnd::OnNMRclickNick(NMHDR *pNMHDR, LRESULT *pResult)
{
	if (nickList.GetSelectionMark() != ( -1))
	{
		CTitleMenu	menuNick;
		POINT		point;

		::GetCursorPos(&point);

		menuNick.CreatePopupMenu();
		menuNick.AddMenuTitle(GetResString(IDS_IRC_NICK));
		menuNick.AppendMenu(MF_STRING, Irc_Priv, GetResString(IDS_IRC_PRIVMESSAGE));
	//	menuNick.AppendMenu(MF_STRING,Irc_AddFriend,GetResString(IDS_IRC_ADDTOFRIENDLIST));
	//	if( !GetSendFileString().IsEmpty() )
	//	menuNick.AppendMenu(MF_STRING,Irc_SendLink, (CString)GetResString(IDS_IRC_SENDLINK) + GetSendFileString() );
		menuNick.AppendMenu(MF_STRING, Irc_Slap, GetResString(IDS_IRC_SLAP));
		menuNick.AppendMenu(MF_STRING, Irc_Owner, _T("Owner"));
		menuNick.AppendMenu(MF_STRING, Irc_DeOwner, _T("DeOwner"));
		menuNick.AppendMenu(MF_STRING, Irc_Op, GetResString(IDS_IRC_OP));
		menuNick.AppendMenu(MF_STRING, Irc_DeOp, GetResString(IDS_IRC_DEOP));
		menuNick.AppendMenu(MF_STRING, Irc_HalfOp, GetResString(IDS_IRC_HALFOP));
		menuNick.AppendMenu(MF_STRING, Irc_DeHalfOp, GetResString(IDS_IRC_DEHALFOP));
		menuNick.AppendMenu(MF_STRING, Irc_Voice, GetResString(IDS_IRC_VOICE));
		menuNick.AppendMenu(MF_STRING, Irc_DeVoice, GetResString(IDS_IRC_DEVOICE));
		menuNick.AppendMenu(MF_STRING, Irc_Protect, _T("Protect"));
		menuNick.AppendMenu(MF_STRING, Irc_DeProtect, _T("DeProtect"));
		menuNick.AppendMenu(MF_STRING, Irc_Kick, GetResString(IDS_IRC_KICK));
		menuNick.TrackPopupMenuEx(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y, this, NULL);

//	Menu objects are destroyed in their destructor
	}
	*pResult = 0;
}

void CIrcWnd::OnNMDblclkNickList(NMHDR *pNMHDR, LRESULT *pResult)
{
	int		iNickItem = nickList.GetSelectionMark();

	if (iNickItem != -1)
	{
		Nick	*pNick = (Nick*)nickList.GetItemData(iNickItem);

		if (pNick != NULL)
			AddMessage(pNick->nick, _T(""), RGB(0, 147, 0), GetResString(IDS_IRC_PRIVATECHANSTART));
	}
	*pResult = 0;
}

/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////
//	Messages
/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////

void CIrcWnd::AddMessage(CString channelName, CString targetname, COLORREF crTextColor, CString line, ...)
{
	va_list		argptr;
	CString		strTemp;
	Channel		*pUpdateChannel = NULL;

	if (!channelName.IsEmpty())
	{
		pUpdateChannel = FindChannelByName(channelName);
		if (pUpdateChannel == NULL)
			pUpdateChannel = NewChannel(channelName, (channelName[0] == _T('#')) ? 4 : 5);
	}
	else
		pUpdateChannel = (Channel*)channelPtrList.GetHead();

	if (pUpdateChannel == NULL)
		return;

	va_start(argptr, line);
	strTemp.FormatV(line, argptr);
	line = strTemp;
	va_end(argptr);

	line.Remove(_T('\022')); // Reverse
	line.Remove(_T('\026'));
	line.Remove(_T('\b'));
	line.Remove(_T('\t'));
	line.Replace(_T('\004'), _T('%'));

	if (channelName.IsEmpty())
	{
		strTemp = m_pIrcMain->GetNick();

		int			iLen = strTemp.GetLength();

		if (iLen > 0 && line.Left(iLen) == strTemp)
		{
			if (line.Mid(iLen + 1, 1) == _T(":"))
				iLen ++;
			line = line.Right(line.GetLength() - (iLen + 1));
		}
	}

	if (line.IsEmpty())
		return;

	COLORREF	crForegroundColor = crTextColor, crBackgroundColor = 0x1000000;
	int			iIndex = 0, iColorIndex;
	DWORD		dwFlags = 0;

	if (g_eMuleApp.m_pGlobPrefs->GetIRCAddTimestamp())
	{
		strTemp = CTime::GetCurrentTime().Format(_T("%X: "));
		pUpdateChannel->log.AppendText(strTemp, RGB(0, 0, 128));
	}

	if (!channelName.IsEmpty() && !targetname.IsEmpty())
		pUpdateChannel->log.AppendText(_T("<") + targetname + _T("> "), (m_pIrcMain->GetNick() == targetname) ? RGB(128, 0, 128) : RGB(51, 51, 153));

	while (!line.IsEmpty())
	{
		switch (line.GetAt(iIndex))
		{
			case 2:
			{
				line.Delete(iIndex, 1);
				if (iIndex > 0)
				{
					pUpdateChannel->log.AppendText(line.Left(iIndex), crForegroundColor, crBackgroundColor, dwFlags);
					line = line.Mid(iIndex);
					iIndex = 0;
				}
				dwFlags ^= HTC_BOLD;
				break;
			}
			case 3:
			{
				line.Delete(iIndex, 1);
				if (iIndex > 0)
				{
					pUpdateChannel->log.AppendText(line.Left(iIndex), crForegroundColor, crBackgroundColor, dwFlags);
					line = line.Mid(iIndex);
					iIndex = 0;
				}

				if (!line.IsEmpty())
				{
					if (line.GetAt(0) >= 48 && line.GetAt(0) <= 58)
					{
						if (line.GetLength() > 1)
						{
							iIndex = 1;
							if (line.GetAt(1) >= 48 && line.GetAt(1) <= 58)
								iIndex++;
							iColorIndex = _tstoi(line.Left(iIndex));
							if (iColorIndex < 16)
								crForegroundColor = _acrColorPal[iColorIndex];
							line = line.Mid(iIndex);
							if (line.GetLength() > 1 && line.GetAt(0) == 44 && line.GetAt(1) >= 48 && line.GetAt(1) <= 58)
							{
								iIndex = 1;
								if (line.GetLength() > 2 && line.GetAt(2) >= 48 && line.GetAt(2) <= 58)
									iIndex++;
								iColorIndex = _tstoi(line.Mid(1, iIndex));
								if (iColorIndex < 16)
									crBackgroundColor = _acrColorPal[iColorIndex];
								line = line.Mid(iIndex + 1);
							}
							iIndex = 0;
						}
						else
							line.Empty();
					}
					else
					{
						crForegroundColor = crTextColor;
						crBackgroundColor = 0x1000000;
					}
				}
				break;
			}
			case 15:
			{
				line.Delete(iIndex, 1);
				if (iIndex > 0)
				{
					pUpdateChannel->log.AppendText(line.Left(iIndex), crForegroundColor, crBackgroundColor, dwFlags);
					line = line.Mid(iIndex);
					iIndex = 0;
				}
				crForegroundColor = crTextColor;
				crBackgroundColor = 0x1000000;
				dwFlags = 0;
				break;
			}
			case 29:
			{
				line.Delete(iIndex, 1);
				if (iIndex > 0)
				{
					pUpdateChannel->log.AppendText(line.Left(iIndex), crForegroundColor, crBackgroundColor, dwFlags);
					line = line.Mid(iIndex);
					iIndex = 0;
				}
				dwFlags ^= HTC_ITALIC;
				break;
			}
			case 31:
			{
				line.Delete(iIndex, 1);
				if (iIndex > 0)
				{
					pUpdateChannel->log.AppendText(line.Left(iIndex), crForegroundColor, crBackgroundColor, dwFlags);
					line = line.Mid(iIndex);
					iIndex = 0;
				}
				dwFlags ^= HTC_UNDERLINE;
				break;
			}
			default:
			{
				iIndex++;
				break;
			}
		}
		if (iIndex >= line.GetLength())
		{
			pUpdateChannel->log.AppendText(line, crForegroundColor, crBackgroundColor, dwFlags);
			pUpdateChannel->log.AppendText(_T("\n"));
			line.Empty();
		}
	}

	if (m_pCurrentChannel != pUpdateChannel)
		SetActivity(pUpdateChannel->name, true);
}

void CIrcWnd::SetConnectStatus(bool flag)
{
	if (flag)
	{
		SetDlgItemText(IDC_BN_IRCCONNECT, GetResString(IDS_IRC_DISCONNECT));
		AddMessage(_T(""), _T(""), 0x1000000, GetResString(IDS_CONNECTED));
		m_bConnected = true;
	}
	else
	{
		SetDlgItemText(IDC_BN_IRCCONNECT, GetResString(IDS_IRC_CONNECT));
		AddMessage(_T(""), _T(""), 0x1000000, GetResString(IDS_DISCONNECTED));
		m_bConnected = false;
		m_bLoggedIn = false;

		Channel		*pToDel = (Channel*)channelPtrList.GetTail();

		while (channelPtrList.GetCount() > 2)
		{
			pToDel = (Channel*)channelPtrList.GetTail();
			RemoveChannel(pToDel->name);
		}
		GetDlgItem(IDC_CLOSECHAT)->EnableWindow(false);
		GetDlgItem(IDC_CHATSEND)->EnableWindow(false);
	}
}

void CIrcWnd::NoticeMessage(CString source, CString message)
{
	bool		bFlag = false;
	Channel		*pCurChannel = FindChannelByName(source);
	Nick		*pCurNick;

	if (pCurChannel != NULL)
	{
		AddMessage(source, _T(""), RGB(153, 51, 102), _T("-%s- %s"), source, message);
		bFlag = true;
	}

	for (POSITION pos = channelPtrList.GetHeadPosition(); pos != NULL;)
	{
		pCurChannel = (Channel*)channelPtrList.GetNext(pos);
		pCurNick = FindNickByName(pCurChannel->name, source);
		if (pCurNick)
		{
			AddMessage(pCurChannel->name, _T(""), RGB(153, 51, 102), _T("-%s- %s"), source, message);
			bFlag = true;
		}
	}
	if (bFlag == false)
	{
		if (m_pCurrentChannel->type == 4)
			AddMessage(m_pCurrentChannel->name, _T(""), RGB(153, 51, 102), _T("-%s- %s"), source, message);
		else
			AddMessage(_T(""), _T(""), RGB(153, 51, 102), _T("-%s- %s"), source, message);
	}
}

//We cannot support color within the text since HyperTextCtrl does not detect hyperlinks with color. So I will filter it.
CString CIrcWnd::StripMessageOfFontCodes(CString temp)
{
	temp = StripMessageOfColorCodes(temp);
	temp.Remove(_T('\002'));
	temp.Remove(_T('\003'));
	temp.Remove(_T('\017'));
	temp.Remove(_T('\026'));
	temp.Remove(_T('\037'));
	temp.Remove(_T('\b'));
	temp.Remove(_T('\t'));
	return temp;
}

CString CIrcWnd::StripMessageOfColorCodes(CString temp)
{
	if (!temp.IsEmpty())
	{
		CString		strTemp1, strTemp2;
		int			iTest = temp.Find(3);

		if (iTest != -1)
		{
			int		iTestLength = temp.GetLength() - iTest;

			if (iTestLength < 2)
				return temp;

			strTemp1 = temp.Left(iTest);
			strTemp2 = temp.Mid(iTest + 2);
			if (iTestLength < 4)
				return strTemp1 + strTemp2;

			if (strTemp2[0] == 44 && strTemp2.GetLength() > 2)
			{
				strTemp2 = strTemp2.Mid(2);
				for (int I = 48; I < 58; I++)
				{
					if (strTemp2[0] == I)
						strTemp2 = strTemp2.Mid(1);
				}
			}
			else
			{
				for (int I = 48; I < 58; I++)
				{
					if (strTemp2[0] == I)
					{
						strTemp2 = strTemp2.Mid(1);
						if (strTemp2[0] == 44 && strTemp2.GetLength() > 2)
						{
							strTemp2 = strTemp2.Mid(2);
							for (int I = 48; I < 58; I++)
							{
								if (strTemp2[0] == I)
									strTemp2 = strTemp2.Mid(1);
							}
						}
					}
				}
			}
			temp = strTemp1 + strTemp2;
			temp = StripMessageOfColorCodes(temp);
		}
	}
	return temp;
}

void CIrcWnd::SetTitle(CString channel, CString title)
{
	Channel		*pCurChannel = FindChannelByName(channel);

	if (pCurChannel == NULL)
		return;

	pCurChannel->title = StripMessageOfFontCodes(title);
	if (pCurChannel == m_pCurrentChannel)
		titleWindow.SetWindowText(pCurChannel->title);
}

void CIrcWnd::SetActivity(CString channel, bool flag)
{
	Channel		*pRefresh = FindChannelByName(channel);

	if (pRefresh == NULL)
	{
		pRefresh = (Channel*)channelPtrList.GetHead();
		if (pRefresh == NULL)
			return;
	}
	TCITEM		tcitem;
	int			i;

	tcitem.mask = TCIF_PARAM;
	tcitem.lParam = -1;
	for (i = 0; i < channelselect.GetItemCount();i++)
	{
		channelselect.GetItem(i, &tcitem);
		if (((Channel*)tcitem.lParam) == pRefresh)
			break;
	}
	if (((Channel*)tcitem.lParam) != pRefresh)
		return;
	/*if (flag)
	{
		item.mask = TCIF_IMAGE;
		item.iImage = 2;
		channelselect.SetItem(i, &item);
	}
	else
	{
		item.mask = TCIF_IMAGE;
		item.iImage = 1;
		channelselect.SetItem(i, &item);
	}*/
	if(channelselect.GetCurSel() != i)
		channelselect.SetItemState(i, TCIS_HIGHLIGHTED, TCIS_HIGHLIGHTED);
}

void CIrcWnd::OnBnClickedChatsend()
{
	CString		strSend, strTemp;

	GetDlgItemText(IDC_INPUTWINDOW, strSend);
	SetDlgItemText(IDC_INPUTWINDOW, _T(""));

	if (m_pCurrentChannel->history.GetCount() == g_eMuleApp.m_pGlobPrefs->GetMaxChatHistoryLines())
		m_pCurrentChannel->history.RemoveAt(0);
	m_pCurrentChannel->history.Add(strSend);
	m_pCurrentChannel->history_pos = m_pCurrentChannel->history.GetCount();

	if (strSend.IsEmpty())
		return;

	if (!this->m_bConnected)
		return;

	if (strSend.Left(4) == _T("/hop"))
	{
		if (m_pCurrentChannel->name.Left(1) == _T("#"))
		{
			strTemp = m_pCurrentChannel->name;
			m_pIrcMain->SendString(_T("PART ") + strTemp );
			m_pIrcMain->SendString(_T("JOIN ") + strTemp );
		}
		return;
	}

	if (strSend.Left(1) == _T("/") && strSend.Left(3).MakeLower() != _T("/me") /*&& strSend.Left(6).MakeLower() != _T("/sound")*/)
	{
		if (strSend.Left(4) == _T("/msg"))
		{
			if (m_pCurrentChannel->type == 4 || m_pCurrentChannel->type == 5)
			{
				strSend.Replace(_T('%'), _T('\004'));
				AddMessage(m_pCurrentChannel->name, _T(""), RGB(153, 51, 102), _T("* >> ") + strSend.Mid(5));
				strSend.Replace(_T('\004'), _T('%'));
			}
			else
			{
				strSend.Replace(_T('%'), _T('\004'));
				AddMessage(_T(""), _T(""), RGB(153, 51, 102), _T("* >> ") + strSend.Mid(5));
				strSend.Replace(_T('\004'), _T('%'));
			}
			strSend = _T("/PRIVMSG") + strSend.Mid(4);
		}
		if (((CString)strSend.Left(17)).CompareNoCase(_T("/PRIVMSG nickserv")) == 0)
			strSend = _T("/ns") + strSend.Mid(17);
		else if (((CString)strSend.Left(17)).CompareNoCase(_T("/PRIVMSG chanserv")) == 0)
			strSend = _T("/cs") + strSend.Mid(17);
		else if (((CString)strSend.Left(8)).CompareNoCase(_T("/PRIVMSG")) == 0 || ((CString)strSend.Left(6)).CompareNoCase(_T("/TOPIC")) == 0)
			strSend.Insert(strSend.Find(_T(" "), strSend.Find(_T(" ")) + 1) + 1, _T(":"));
		m_pIrcMain->SendString(strSend.Mid(1));
		return;
	}
	if (m_pCurrentChannel->type < 4)
	{
		m_pIrcMain->SendString(strSend);
		return;
	}
	if (strSend.Left(3) == _T("/me"))
	{
		strTemp.Format(_T("PRIVMSG %s :\001ACTION %s\001"), m_pCurrentChannel->name, strSend.Mid(4));
		strSend.Replace(_T('%'), _T('\004'));
		AddMessage(m_pCurrentChannel->name, _T(""), RGB(0, 147, 0), _T("* %s %s"), m_pIrcMain->GetNick(), strSend.Mid(4));
		m_pIrcMain->SendString(strTemp);
		return;
	}
	strTemp = _T("PRIVMSG ") + m_pCurrentChannel->name + _T(" :") + strSend;
	m_pIrcMain->SendString(strTemp);
	strSend.Replace(_T('%'), _T('\004'));
	AddMessage(m_pCurrentChannel->name, m_pIrcMain->GetNick(), 0x1000000, strSend);
	strSend.Replace(_T('\004'), _T('%'));
}

/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////
//	Channels
/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////

Channel* CIrcWnd::FindChannelByName(CString name)
{
	Channel		*pCurChannel;

	for (POSITION pos = channelPtrList.GetHeadPosition(); pos != NULL;)
	{
		pCurChannel = (Channel*)channelPtrList.GetNext(pos);
		if (pCurChannel->name.CompareNoCase(name.Trim()) == 0 && (pCurChannel->type == 4 || pCurChannel->type == 5))
			return pCurChannel;
	}
	return 0;
}

Channel* CIrcWnd::NewChannel(CString name, byte type)
{
	Channel		*pToAdd = new Channel;
	TCITEM		tcitem;

	pToAdd->name = name;
	pToAdd->title = name;
	pToAdd->type = type;
	pToAdd->history_pos = 0;
	if (type != 2)
	{
		CRect		rcChannel;

		serverChannelList.GetWindowRect(&rcChannel);
		pToAdd->log.Create(WS_CHILD | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_HSCROLL | WS_VSCROLL | ES_MULTILINE | ES_READONLY | ES_NOHIDESEL, rcChannel, this, (UINT)-1);
		pToAdd->log.ModifyStyleEx(0, WS_EX_STATICEDGE, SWP_FRAMECHANGED);
		pToAdd->log.SendMessage(EM_SETMARGINS, EC_LEFTMARGIN | EC_RIGHTMARGIN, MAKELONG(3, 3));
		pToAdd->log.SetEventMask(pToAdd->log.GetEventMask() | ENM_LINK);
		pToAdd->log.SetFont(&g_eMuleApp.m_pdlgEmule->m_fontDefault);
		pToAdd->log.SetTitle(name);
		pToAdd->log.m_bWordWrap = true;
		pToAdd->log.SetTargetDevice(NULL, 0);
	}
	channelPtrList.AddTail(pToAdd);

	tcitem.mask = TCIF_PARAM | TCIF_TEXT;
	if (type >= 4)
		tcitem.mask |= TCIF_IMAGE;
	tcitem.lParam = (LPARAM)pToAdd;
	tcitem.pszText = name.GetBuffer();
	tcitem.cchTextMax = (int)name.GetLength() + 1;
	tcitem.iImage = 1;
	uint32 pos = channelselect.GetItemCount();

	channelselect.InsertItem(pos, &tcitem);
	if (type >= 4)
	{
		channelselect.SetCurSel(pos);
		channelselect.SetCurFocus(pos);
		OnTcnSelchangeTab2( NULL, NULL );
		GetDlgItem(IDC_CLOSECHAT)->EnableWindow(true);
		GetDlgItem(IDC_CHATSEND)->EnableWindow(true);
	}

	return pToAdd;
}

void CIrcWnd::RemoveChannel(CString channel)
{
	Channel		*pToDel = FindChannelByName(channel);

	if (pToDel == NULL)
		return;

	TCITEM		tcitem;
	int			i;

	tcitem.mask = TCIF_PARAM;
	tcitem.lParam = -1;
	
	for (i = 0; i < channelselect.GetItemCount();i++)
	{
		channelselect.GetItem(i, &tcitem);
		if (((Channel*)tcitem.lParam) == pToDel)
			break;
	}
	
	if (((Channel*)tcitem.lParam) != pToDel)
		return;

	channelselect.DeleteItem(i);

	if (pToDel == m_pCurrentChannel)
	{
		nickList.DeleteAllItems();
		if (channelselect.GetItemCount() > 2 && i > 1)
		{
			if (i == 2)
				i++;
			channelselect.SetCurSel(i - 1);
			channelselect.SetCurFocus(i - 1);
			OnTcnSelchangeTab2(NULL, NULL);
		}
		else
		{
			channelselect.SetCurSel(0);
			channelselect.SetCurFocus(0);
			OnTcnSelchangeTab2(NULL, NULL);
			GetDlgItem(IDC_CLOSECHAT)->EnableWindow(false);
			GetDlgItem(IDC_CHATSEND)->EnableWindow(false);
		}
	}
	DeleteAllNick(pToDel->name);
	channelPtrList.RemoveAt(channelPtrList.Find(pToDel));
	delete pToDel;
}

void CIrcWnd::DeleteAllChannel()
{
	POSITION	pos1, pos2;
	Channel		*pChannel;

	for (pos1 = channelPtrList.GetHeadPosition();(pos2 = pos1) != NULL;)
	{
		pChannel = (Channel*)channelPtrList.GetNext(pos1);
		DeleteAllNick(pChannel->name);
		channelPtrList.RemoveAt(pos2);
		delete pChannel->log;
		delete pChannel;
	}
}

void CIrcWnd::JoinChannels()
{
	if (!this->m_bConnected)
		return;

	int			iIndex = -1;
	POSITION	pos = serverChannelList.GetFirstSelectedItemPosition();

	while (pos != NULL)
	{
		iIndex = serverChannelList.GetNextSelectedItem(pos);
		if (iIndex > -1)
			m_pIrcMain->SendString(_T("JOIN ") + serverChannelList.GetItemText(iIndex, 0));
	}
}

LRESULT CIrcWnd::OnCloseTab(WPARAM wparam, LPARAM lparam)
{
	OnBnClickedClosechat((int)wparam);
	return true;
}

void CIrcWnd::SendString(CString send)
{
	if (this->m_bConnected)
		m_pIrcMain->SendString(send);
}

void CIrcWnd::ScrollHistory(bool down)
{
	if ((m_pCurrentChannel->history_pos == 0 && !down) || (m_pCurrentChannel->history_pos == m_pCurrentChannel->history.GetCount() && down))
		return;

	if (down)
		++m_pCurrentChannel->history_pos;
	else
		--m_pCurrentChannel->history_pos;

	CString		strBuffer;
	
	if (m_pCurrentChannel->history_pos != m_pCurrentChannel->history.GetCount())
		strBuffer = m_pCurrentChannel->history.GetAt(m_pCurrentChannel->history_pos);

	SetDlgItemText(IDC_INPUTWINDOW, strBuffer);
	inputWindow.SetSel(strBuffer.GetLength(), strBuffer.GetLength());
}
