//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#pragma once
#include "stdafx.h"
#include "KnownFile.h"
#include "types.h"
#include "opcodes.h"
#include "BarShader.h"
#include "sourcesaver.h"
#include "PastComment.h"
#include "Category.h"

//	TEMP - MOREVIT: I don't think the length of the download queue accurately gives the
//		number of transferring sources.
#define USE_TRANSFERRINGSRCCOUNT	1

enum _EnumPartFileStatuses
{
	PS_READY			= 0,
	PS_EMPTY			= 1,
	PS_WAITINGFORHASH	= 2,
	PS_HASHING			= 3,
	PS_ERROR			= 4,
	PS_UNKNOWN			= 6,
	PS_PAUSED			= 7,
	PS_COMPLETING		= 8,
	PS_COMPLETE			= 9,
	PS_STOPPED			= 10,	//eklmn: added stop status
	PS_DOWNLOADING		= 20,	//MOREVIT - Dummy status
	PS_WAITINGFORSOURCE	= 21,	//MOREVIT - Dummy status
	PS_STALLED			= 22,	//MOREVIT - Dummy status
	PS_ACTIVE			= 23,	//MOREVIT - Dummy status = Downloading || on queue
};
typedef EnumDomain<_EnumPartFileStatuses>	EnumPartFileStatuses;

//	The number of days without a transfer before a download is considered stalled
#define WARN_PERIOD_OF_NO_PROGRESS 7

#define BUFFER_TIME_LIMIT	180000   // Max milliseconds before forcing a flush // netwolf: increased to 3 minutes (be nice to your HDD :))

class CSearchFile;
class CUpDownClient;
class CClientSource;

struct PartFileBufferedData
{
	BYTE					   *m_pbyteBuffer;		// Barry - This is the data to be written
	uint32						m_dwStartOffset;	// Barry - This is the start offset of the data
	uint32						m_dwEndOffset;		// Barry - This is the end offset of the data
};

class CPartFile : public CKnownFile
{
public:
	CPartFile();
	CPartFile(CSearchFile* searchresult);
	CPartFile(CString edonkeylink);
	CPartFile(class CED2KFileLink* fileLink);
	void	InitializeFromLink(CED2KFileLink* fileLink);
	virtual	~CPartFile();

	bool	CreateFromFile(char* directory,char* filename)	{return false;}
	bool	LoadFromFile(FILE* file)						{return false;}
	bool	WriteToFile(FILE* file)							{return false;}
	bool	IsPartFile()									{return !(m_eStatus == PS_COMPLETE);}
	uint32	Process(uint32 reducedownload, uint32 iteration);
	bool	LoadPartFile(LPCTSTR in_directory, LPCTSTR filename);
	bool	SavePartFile();
	void	PartFileHashFinished(CKnownFile* result);
	bool	HashSinglePart(uint16 partnumber); // true = ok , false = corrupted
	uint64	GetRealFileSize();

	void	AddGap(uint32 start, uint32 end);
	void	FillGap(uint32 start, uint32 end);
	void	DrawStatusBar(CDC* dc, RECT* rect, bool bFlat);
	bool	IsComplete(uint32 start, uint32 end);
	bool	IsCorruptedPart(uint16 partnumber);
	void	UpdateCompletedInfos();
	void	GetGapsInPart(CUpDownClient* sender, uint16 numPart, Requested_Block_Struct** newblocks, uint16* count); //jicxicmic
	uint16	GetNextRequestedBlock(CUpDownClient* sender);
	void	WritePartStatus(CFile* file);
	bool	HasParts();
	void	WriteCompleteSourcesCount(CFile* file);
	void	AddSources(CMemFile& pSources, uint32 iServerIP, uint16 iServerPort);
	EnumPartFileStatuses	GetStatus(bool ignorepause = false);
	void	NewSrcPartsInfo();
	void	SetPriority(byte np);
	CString	GetPartMetFileName() const				{return m_strPartMetFileName;}
	uint32	GetTransferred() const					{return m_dwBytesTransferred;}
	void	AddRxAmount(uint32 dwRxBytes)			{ m_dwBytesTransferred += dwRxBytes; }
	void	AddRxCorruptedAmount(uint32 dwBadBytes)	{m_qwLostDueToCorruption += dwBadBytes; m_qwSessionLostDueToCorruption += dwBadBytes;}
	byte	GetPriority()							{return priority;}
	CString	GetFullName() const						{return m_strFullName;}
	CString	GetTempDir() const						{return m_strTempDir;}
	uint16	GetSourceCount();
	uint32	GetDataRate()							{return m_dataRate;}
#if USE_TRANSFERRINGSRCCOUNT
	uint16	GetTransferringSrcCount()				{ return m_iNumTransferringSrcs; }
#else
	uint16	GetTransferringSrcCount()				{return m_downloadingSourceList.GetCount();}
#endif
	uint16	GetNoNeededPartsSrcCount()				{return m_iSrcNNP;}
	uint16	GetOnQueueSrcCount()					{return m_iSrcOnQueue;}
	uint16	GetHighQRSrcCount()						{return m_iSrcHighQR;}
	uint16	GetConnectingSrcCount()					{return m_iSrcConnecting;}
	uint16	GetTooManyConnsSrcCount()				{return m_iSrcTooManyConns;}
	uint16	GetConnectedSrcCount()					{return m_iSrcConnected;}
	uint16	GetTimeoutedSrcCount()					{return m_iSrcTimeout ;}
	uint16	GetConnectingViaServerSrcCount()		{return m_iSrcConnViaServer;}
	uint16	GetUnknownSrcCount()					{return m_iSrcUnknown;}
	uint16	GetRecHashsetSrcCount()					{return m_iSrcRecHash;}
	uint16	GetLow2LowSrcCount()					{return m_iSrcLowToLow;}
	uint16	GetQueueFullSrcCount()					{return m_iSrcQueueFull;}
	uint16	GetNoUploadSrcCount()					{return m_iSrcNoUpload;}
	uint16	GetBannedSrcCount()						{return m_iSrcBanned;}
	uint16	GetErrorSrcCount()						{return m_iSrcError;}
	uint16	GetSrcA4AFCount()						{return m_iSrcA4AF;}
	uint16  GetNotCurrentSourcesCount();
	int		GetValidSourcesCount();
	uint16	GetCompleteSourcesCount()				{return m_nLastCompleteSrcCount;}

	double	GetPercentCompleted()					{return m_dblPercentCompleted;}
	void	GetCompleteSourcesRange(uint16 *lo, uint16 *hi)	{*lo= *hi= GetCompleteSourcesCount();}
	bool	IsMovie();
	bool 	IsAviMovie();
	bool 	IsMpgMovie();
	bool 	IsMovieChunk(byte mode,uint16 chunk);
	bool 	IsBetterMovieChunk(uint16 oldchunk,uint16 newchunk);
	bool 	Movie1(byte mode,uint16 oldchunk,uint16 newchunk);
	bool 	Movie2(byte mode,byte raremovie,bool forcepreview,uint16 oldchunk,uint16 newchunk);
	byte 	PrepareComparePart(uint16 chunk);
	bool	IsArchive(bool onlyPreviewable=false);
	CString GetPartFileStatus();
	int		GetPartFileStatusID();
	sint32	GetTimeRemaining(bool bAvgerage = false);
	CTime	lastseencomplete;
	int		GetPartFileStatusRang();
	void	SetStartTimeReset(bool bEnable)		{m_StartTimeReset = bEnable;}
	bool	GetStartTimeReset()					{return m_StartTimeReset;}
	uint32	GetAvgDataRate(bool bUpdated = false);
	uint32	GetSessionTransferred()				{return (completedsize > m_SessionStartSize ? completedsize - m_SessionStartSize : 0);}
	CTimeSpan GetFlushTimeSpan()				{return (m_timeLastDownTransfer - m_SessionStartTime);}
	CTimeSpan GetSessionTimeSpan()				{return (CTime::GetCurrentTime() - m_SessionStartTime);}
	CFile	&GetPartFileHandle()				{return m_hPartFileRead;}

	uint32	WriteToBuffer(sint32 iCompressionGain, BYTE *data, uint32 start, uint32 end, Requested_Block_Struct *block, bool bFreeBuff);
	void	FlushBuffer(void);

	// Barry - This will invert the gap list, up to caller to delete gaps when done
	// 'Gaps' returned are really the filled areas, and guaranteed to be in order
	void	GetFilledList(CTypedPtrList<CPtrList, Gap_Struct*> *filled);

	// Barry - Is archive recovery in progress
	volatile bool m_bRecoveringArchive;
	// Barry - Added to prevent list containing deleted blocks on shutdown
	void	RemoveAllRequestedBlocks(void);

	uint16  CPartFile::GetRemainingBlocksInPart(uint16 partnumber);
	void	RemoveBlockFromList(uint32 start,uint32 end);
	void	RemoveAllSources(bool bTryToSwap);
	void	DeleteFile();
	void	StopFile(bool bCancel = true);
	bool	IsPaused() const				{return m_bPaused;}
	bool	IsStopped() const				{return m_bStopped;}
	bool	IsStalled() const;
	bool	IsCompleting() const			{return (m_eStatus == PS_COMPLETING);}
	void	PauseFile();
	void	ResumeFile();
	void	GetSourcesAfterServerConnect();

	virtual	Packet* CreateSrcInfoPacket(CUpDownClient* ForClient);
	void	AddClientSources(CMemFile* sources, byte sourceexchangeversion = 1);

	void	PreviewFile();
	bool	PreviewAvailable();
	bool	IsVLCInstalled();
	uint32	GetAvailablePartCount() const	{return m_dwAvailablePartsCount;}

	uint32	GetLastAnsweredTime()			{ return m_ClientSrcAnswered; }
	void	SetLastAnsweredTime()			{ m_ClientSrcAnswered = ::GetTickCount(); }
	void	SetLastAnsweredTimeTimeout()	{ m_ClientSrcAnswered = 2 * CONNECTION_LATENCY + ::GetTickCount() - SOURCECLIENTREASK; }
	uint64	GetLostDueToCorruption() const			{return m_qwLostDueToCorruption;}
	uint64	GetGainDueToCompression() const			{return m_iGainDueToCompression;}
	uint64	GetSessionLostDueToCorruption() const	{return m_qwSessionLostDueToCorruption;}
	uint64	GetSessionGainDueToCompression() const	{return m_iSessionGainDueToCompression;}
	uint32	TotalPacketsSavedDueToICH() const		{return m_iTotalPacketsSavedDueToICH;}
	bool	HasComment() const						{return hasComment;}
	bool	HasRating() const						{return hasRating;}
	EnumPartFileRating		GetRating();
	void	SetHasComment(bool in)			{hasComment=in;}
	void	SetHasRating(bool in)			{hasRating=in;}
	void	UpdateFileRatingCommentAvail();

	uint32	GetCommonFilePenalty();
	void	UpdateDisplayedInfo(boolean bForce=false);

	void	GetSizeToTransfer(uint32& pui32SizeToTransfer);

	byte*	MMCreatePartStatus();

	volatile bool m_bPreviewing;

	uint16	GetSrcPartFrequency(uint16 part) { return m_srcPartFrequencies[part]; }

	void	UpdateDownloadAutoPriority(void);
	bool	IsAutoPrioritized() 					{ return m_bAutoPriority ; }
	void	SetAutoPriority(bool newAutoPriority) 	{ m_bAutoPriority = newAutoPriority ; }
	void	RemoveNoNeededSources(uint32 number_sources_to_remove = 0xffffffff);

	void	LoadSettingsFile();
	void	SaveSettingsFile();

	EnumCategories	GetCatID() const;
	void			SetCatID(EnumCategories eCatID);

	void	AllocateNeededSpace();
	bool	IsLastBlockComplete();
	bool	IsPreallocated() 						{ return m_bIsPreallocated; }
	static DWORD WINAPI AllocateNeededSpaceProc(LPVOID lpParameter);

	void	AddPastComment(CUpDownClient *pClient);
	void	RemovePastComment(CUpDownClient *pClient, bool bRestore= true);
	CPastCommentList &GetPastCommentList()			{ return m_pastCommentList; }
	int		ReadFileForUpload(uint32 dwOffset, uint32 dwBytesToRead, byte *pbyteBuffer);

	bool	SavePartFileStats();
	uint32	GetCompletedSize()   {return completedsize;}
	CTime	GetLastDownTransfer() const {return m_timeLastDownTransfer;};
	void	AddClientSources(CTypedPtrList<CPtrList, CClientSource*>*);
	void	AddClientSource(CClientSource* pSource, int iSource=0, bool bExchanged=false, byte byteSourceExchangeVersion = 0);
	void	AddClientHostnameSources(CTypedPtrList<CPtrList, CClientSource*>);
	CString	CreateED2KSourceLink(uint32 expirein, int sourcenum, bool bIsSourceSaver = false);
	void	SetAlternativeOutputDir(CString* path);
	CString	GetOutputDir();

	CString	LocalizeLastSeenComplete()	{return DtLocale(lastseencomplete);}
	CString	LocalizeLastDownTransfer()	{return DtLocale(m_timeLastDownTransfer);}

	CString GetProgressString(uint16 size);
	void 	CharFillRange(CString* buffer,double start, double end, char color);
	CString	GetDownloadFileInfo();
	CString	GetDownloadFileInfo4Tooltips();
	void 	DownloadAllA4AF();
	void	AddDownloadingSource(CUpDownClient* client);
	void	RemoveDownloadingSource(CUpDownClient* client);
	uint32 	GetBytesInPart(uint16 partnumber, bool left2DL = false, bool dontRecalculate = false);
	void	SetStatus(EnumPartFileStatuses eNewStatus);

	uint32	GetLastPurgeTime(){ return lastpurgetime; }

	bool	IsFakesDotRar() const						{ return m_bIsFakesDotRar; }
	void	SetFakesDotRar(bool bIsFakesDotRar = true)	{ m_bIsFakesDotRar = bIsFakesDotRar; }

protected:
	bool	GetNextEmptyBlockInPart(uint16 partnumber,Requested_Block_Struct* result);
	bool	IsAlreadyRequested(uint32 start, uint32 end);
	void	CompleteFile(bool hashingdone);
	void	CreatePartFile();
	void	Init();

	CSourceSaver		m_sourcesaver;

private:
//	Separate read and write handles to eliminate I/O collisions
	CFile	m_hPartFileWrite;
	CFile	m_hPartFileRead;

	EnumCategories	m_eCategoryID;
	bool			m_bDataFlushReq;
	uint16			count;
//	Statistic
//	We don't need to count transferring sources, because we can get them over m_downloadingSourceList.GetCount()
	uint16	m_iNumTransferringSrcs;
	uint16	m_iSrcNNP;
	uint16	m_iSrcOnQueue;
	uint16	m_iSrcHighQR;
	uint16	m_iSrcConnecting;
	uint16	m_iSrcTooManyConns;
	uint16	m_iSrcConnected;
	uint16	m_iSrcTimeout;
	uint16	m_iSrcConnViaServer;
	uint16	m_iSrcUnknown;
	uint16	m_iSrcRecHash;
	uint16	m_iSrcLowToLow;
	uint16	m_iSrcQueueFull;
	uint16	m_iSrcBanned;
	uint16	m_iSrcNoUpload;
	uint16	m_iSrcError;
	uint16	m_iSrcA4AF;

	uint32	completedsize;
	uint64	m_qwLostDueToCorruption;
	uint64	m_iGainDueToCompression;
	uint64	m_qwSessionLostDueToCorruption;
	uint64	m_iSessionGainDueToCompression;
	uint32	m_iTotalPacketsSavedDueToICH;
	uint32	m_dataRate;
	CString	m_strFullName;
	CString	m_strTempDir;
	CString	m_strPartMetFileName;
	uint32	m_dwBytesTransferred;
	bool	m_bPaused;
	bool	m_bStopped;
	byte	priority;
	EnumPartFileStatuses	m_eStatus;
	uint32	m_dwLastFileSourcesRequestTime;
	uint32	lastpurgetime;
	uint32	m_LastNoNeededCheck;

	CTypedPtrList<CPtrList, Gap_Struct*> gaplist;
	CTypedPtrList<CPtrList, Requested_Block_Struct*> requestedblocks_list;
	CArray<uint16,uint16> m_srcPartFrequencies;
	uint16	m_MaxSrcPartFrequency;
	CArray<uint32,uint32> m_ReqParts;
	CArray<int,int> m_BlockedParts;
	CRITICAL_SECTION					m_csDownloadingSourceList;
	CTypedPtrList<CPtrList, CUpDownClient*> m_downloadingSourceList;
	double	m_dblPercentCompleted;
	CList<uint16,uint16>	corrupted_list;
	uint32	m_dwAvailablePartsCount;
	uint32	m_ClientSrcAnswered;
	uint32	m_nSavedReduceDownload;
	bool	m_bPercentUpdated;
	static	CBarShader s_LoadBar;
	static	CBarShader s_ChunkBar;
	bool	m_bAutoPriority;
	bool	m_bIsBeingDeleted;
	bool	m_bIsPreallocated;

	CTime	m_SessionStartTime;
	uint32	m_SessionStartSize;
	uint32	m_AvgDataRate;
	double	m_dblPercentConfirmed;
	uint32	confirmedsize;
	bool	m_StartTimeReset;

	bool	m_bIsFakesDotRar;
	bool	hasRating;
	bool	hasComment;
	DWORD	m_dwLastRefreshedDLDisplayTime;

	CTypedPtrList<CPtrList, PartFileBufferedData*> m_BufferedData_list;
	uint32	m_nTotalBufferData;
	uint32	m_nLastBufferFlushTime;
	uint16	m_nLastCompleteSrcCount;

	CCriticalSection	m_csSavePartFile;
	CCriticalSection	m_csGapListAndPartStatus;
	CCriticalSection	m_csFileCompletion;

	CTime	m_timeLastDownTransfer;
	CString	m_strAlternativePath;

	CPastCommentList m_pastCommentList;

private:
	CString DtLocale(CTime dt);
	BOOL	PerformFileComplete();
	static UINT CompleteThreadProc(CPartFile* pFile);
	CString GetStatsFullPath() const;
	bool	LoadPartFileStats();
	void	RemovePartFileStats();
	void	ClosePartFile();
	void	UpdateGUIAfterStateChange();

public:
	CTypedPtrList<CPtrList, CUpDownClient*> m_srcLists[SOURCESSLOTS];
	CTypedPtrList<CPtrList, CUpDownClient*> m_A4AFsrcList;

	bool	m_bHashSetNeeded;
};

struct PAThreadParam
{
	uint32		iPosition;
	CPartFile*	cPartFile;
};
